# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    DNSAPIError,
)

if t.TYPE_CHECKING:
    from collections.abc import Sequence  # pragma: no cover

    from ._argspec import OptionProvider  # pragma: no cover
    from ._provider import ProviderInformation  # pragma: no cover
    from ._record import RecordIDT  # pragma: no cover
    from ._record_set import (
        DNSRecordSet,
        IDNSRecordSet,
        RecordSetIDT,
    )  # pragma: no cover
    from ._zone import ZoneIDT  # pragma: no cover
    from ._zone_record_set_api import ZoneRecordSetAPI  # pragma: no cover


def bulk_apply_changes(
    api: ZoneRecordSetAPI[ZoneIDT, RecordSetIDT, RecordIDT],
    provider_information: ProviderInformation,
    options: OptionProvider,
    zone_id: ZoneIDT,
    record_sets_to_delete: (
        Sequence[DNSRecordSet[RecordSetIDT, RecordIDT]] | None
    ) = None,
    record_sets_to_change: (
        Sequence[tuple[DNSRecordSet[RecordSetIDT, RecordIDT], bool, bool]] | None
    ) = None,
    record_sets_to_create: (
        Sequence[IDNSRecordSet[RecordSetIDT | None, RecordIDT | None]] | None
    ) = None,
    stop_early_on_errors: bool = True,
) -> tuple[
    bool,
    list[
        tuple[
            t.Literal["create"],
            IDNSRecordSet[RecordSetIDT | None, RecordIDT | None],
            DNSAPIError,
        ]
        | tuple[
            t.Literal["delete", "change"],
            DNSRecordSet[RecordSetIDT, RecordIDT],
            DNSAPIError,
        ]
    ],
    dict[str, list[DNSRecordSet[RecordSetIDT, RecordIDT]]],
]:
    """
    Update multiple records. If an operation failed, raise a DNSAPIException.

    @param api: A ZoneRecordSetAPI instance
    @param provider_information: A ProviderInformation object.
    @param options: A object compatible with ModuleOptionProvider that gives access to the module/plugin
                    options.
    @param zone_id: Zone ID to apply changes to
    @param record_sets_to_delete: Optional list of DNS records to delete (DNSRecordSet)
    @param record_sets_to_change: Optional list of tuples (DNS records, value changed, TTL changed) to change
                                  (tuple[DNSRecordSet, bool, bool])
    @param record_sets_to_create: Optional list of DNS records to create (DNSRecordSet)
    @param bulk_threshold: Minimum number of changes for using the bulk API instead of the regular API
    @param stop_early_on_errors: If set to ``True``, try to stop changes after the first error happens.
                                 This might only work on some APIs.
    @return A tuple (changed, errors, success) where ``changed`` is a boolean which indicates whether a
            change was made, ``errors`` is a list of tuples (what, record_set, err) for the errors occurred,
            and ``success`` is a dictionary with three lists ``success['deleted']``,
            ``success['changed']`` and ``success['created']``, which list all record sets that were deleted,
            changed and created, respectively.
    """
    record_sets_to_delete = record_sets_to_delete or []
    record_sets_to_change = record_sets_to_change or []
    record_sets_to_create = record_sets_to_create or []

    has_change = False
    errors: list[
        tuple[
            t.Literal["create"],
            IDNSRecordSet[RecordSetIDT | None, RecordIDT | None],
            DNSAPIError,
        ]
        | tuple[
            t.Literal["delete", "change"],
            DNSRecordSet[RecordSetIDT, RecordIDT],
            DNSAPIError,
        ]
    ] = []

    bulk_threshold = 2
    if provider_information.supports_bulk_actions():
        bulk_threshold = options.get_option("bulk_operation_threshold")

    success: dict[str, list[DNSRecordSet[RecordSetIDT, RecordIDT]]] = {
        "deleted": [],
        "changed": [],
        "created": [],
    }

    # Delete record sets
    if len(record_sets_to_delete) >= bulk_threshold:
        results = api.delete_record_sets(
            {zone_id: record_sets_to_delete}, stop_early_on_errors=stop_early_on_errors
        )
        result = results.get(zone_id) or []
        for record_set, deleted, failed in result:
            has_change |= deleted
            if failed is not None:
                errors.append(("delete", record_set, failed))
            if deleted:
                success["deleted"].append(record_set)
        if errors and stop_early_on_errors:
            return has_change, errors, success
    else:
        for record_set in record_sets_to_delete:
            try:
                deleted = api.delete_record_set(zone_id, record_set)
                has_change |= deleted
                if deleted:
                    success["deleted"].append(record_set)
            except DNSAPIError as e:
                errors.append(("delete", record_set, e))
                if stop_early_on_errors:
                    return has_change, errors, success

    # Change record sets
    if len(record_sets_to_change) >= bulk_threshold:
        results = api.update_record_sets(
            {zone_id: record_sets_to_change}, stop_early_on_errors=stop_early_on_errors
        )
        result = results.get(zone_id) or []
        for record_set, changed, failed in result:
            has_change |= changed
            if failed is not None:
                errors.append(("change", record_set, failed))
            if changed:
                success["changed"].append(record_set)
        if errors and stop_early_on_errors:
            return has_change, errors, success
    else:
        for record_set, updated_value, updated_ttl in record_sets_to_change:
            try:
                record_set = api.update_record_set(
                    zone_id,
                    record_set,
                    updated_records=updated_value,
                    updated_ttl=updated_ttl,
                )
                has_change = True
                success["changed"].append(record_set)
            except DNSAPIError as e:
                errors.append(("change", record_set, e))
                if stop_early_on_errors:
                    return has_change, errors, success

    # Create record sets
    if len(record_sets_to_create) >= bulk_threshold:
        create_results = api.add_record_sets(
            {zone_id: record_sets_to_create}, stop_early_on_errors=stop_early_on_errors
        )
        create_result = create_results.get(zone_id) or []
        for crecord_set, created, failed in create_result:
            has_change |= created
            if failed is not None:
                errors.append(("create", crecord_set, failed))
            if created:
                success["created"].append(crecord_set)  # type: ignore  # if created is True, crecord is a DNSRecordSet
        if errors and stop_early_on_errors:
            return has_change, errors, success
    else:
        for create_record_set in record_sets_to_create:
            try:
                created_record_set = api.add_record_set(zone_id, create_record_set)
                has_change = True
                success["created"].append(created_record_set)
            except DNSAPIError as e:
                errors.append(("create", create_record_set, e))
                if stop_early_on_errors:
                    return has_change, errors, success

    return has_change, errors, success



# Copyright (c) 2017-2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.dns.plugins.module_utils._argspec import (
    ArgumentSpec,
    ModuleOptionProvider,
)
from ansible_collections.community.dns.plugins.module_utils._conversion.base import (
    DNSConversionError,
)
from ansible_collections.community.dns.plugins.module_utils._conversion.converter import (
    RecordConverter,
)
from ansible_collections.community.dns.plugins.module_utils._options import (
    create_bulk_operations_argspec,
    create_record_transformation_argspec,
)
from ansible_collections.community.dns.plugins.module_utils._record import (
    DNSRecord,
    format_records_for_output,
)
from ansible_collections.community.dns.plugins.module_utils._record_set import (
    DNSRecordSet,
    format_record_set_for_output,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_api import (
    NOT_PROVIDED,
    DNSAPIAuthenticationError,
    DNSAPIError,
    ZoneRecordAPI,
    filter_records,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_helpers import (
    bulk_apply_changes,
)
from ansible_collections.community.dns.plugins.module_utils._zone_record_set_api import (
    filter_record_sets,
)

from ._utils import get_prefix, normalize_dns_name

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover

    from .._provider import ProviderInformation  # pragma: no cover
    from .._record import IDNSRecord, RecordIDT  # pragma: no cover
    from .._record_set import IDNSRecordSet, RecordSetIDT  # pragma: no cover
    from .._zone import ZoneIDT  # pragma: no cover
    from .._zone_record_set_api import ZoneRecordSetAPI  # pragma: no cover


def create_module_argument_spec(
    provider_information: ProviderInformation,
) -> ArgumentSpec:
    return (
        ArgumentSpec(
            argument_spec={
                "state": {
                    "type": "str",
                    "choices": ["present", "absent"],
                    "required": True,
                },
                "zone_name": {"type": "str", "aliases": ["zone"]},
                "zone_id": {"type": provider_information.get_zone_id_type()},
                "record": {"type": "str"},
                "prefix": {"type": "str"},
                "ttl": {
                    "type": "int",
                    "default": provider_information.get_record_default_ttl(),
                },
                "type": {
                    "choices": provider_information.get_supported_record_types(),
                    "required": True,
                },
                "value": {"type": "list", "elements": "str"},
                "on_existing": {
                    "type": "str",
                    "default": "replace",
                    "choices": ["replace", "keep_and_fail", "keep_and_warn", "keep"],
                },
            },
            required_one_of=[
                ("zone_name", "zone_id"),
                ("record", "prefix"),
            ],
            mutually_exclusive=[
                ("zone_name", "zone_id"),
                ("record", "prefix"),
            ],
            required_if=[
                ("state", "present", ["value"]),
                ("on_existing", "keep_and_fail", ["value"]),
                ("on_existing", "keep_and_warn", ["value"]),
                ("on_existing", "keep", ["value"]),
            ],
        )
        .merge(create_bulk_operations_argspec(provider_information))
        .merge(create_record_transformation_argspec())
    )


def _run_module_record_api(
    option_provider,
    module: AnsibleModule,
    provider_information: ProviderInformation,
    record_converter: RecordConverter,
    record_in: str,
    prefix_in: str | None,
    type_in: str,
    api: ZoneRecordAPI[ZoneIDT, RecordIDT],
) -> t.NoReturn:
    # Get zone information
    if module.params.get("zone_name") is not None:
        zone_in = normalize_dns_name(module.params.get("zone_name"))
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone = api.get_zone_with_records_by_name(
            zone_in, prefix=prefix, record_type=type_in
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_id = zone.zone.id
        records = zone.records
    elif record_in is not None:
        zone = api.get_zone_with_records_by_id(
            module.params.get("zone_id"),
            record_type=type_in,
            prefix=(
                provider_information.normalize_prefix(prefix_in)
                if prefix_in is not None
                else NOT_PROVIDED
            ),
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_in = normalize_dns_name(zone.zone.name)
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone_id = zone.zone.id
        records = zone.records
    else:
        zone_id = module.params.get("zone_id")
        prefix = provider_information.normalize_prefix(prefix_in)
        records = api.get_zone_records(
            zone_id,
            record_type=type_in,
            prefix=prefix,
        )
        if records is None:
            module.fail_json(msg="Zone not found")
        zone_in = None
        record_in = None

    # Find matching records
    records = filter_records(records, prefix=prefix)
    record_converter.process_multiple_from_api(records)

    # Parse records
    values = []
    value_in = module.params.get("value") or []
    value_in = record_converter.process_values_from_user(type_in, value_in)
    values = value_in[:]

    # Compare records
    ttl_in = module.params.get("ttl")
    mismatch = False
    mismatch_records = []
    keep_records = []
    for record in records:
        if record.ttl != ttl_in:
            mismatch = True
            mismatch_records.append(record)
            continue
        val = record.target
        if val in values:
            values.remove(val)
            keep_records.append(record)
        else:
            mismatch = True
            mismatch_records.append(record)
            continue
    if values and module.params.get("state") == "present":
        mismatch = True

    before = [record.clone() for record in records]
    after: list[IDNSRecord[RecordIDT | None]] = keep_records[:]  # type: ignore

    # Determine what to do
    to_create = []
    to_delete = []
    to_change = []
    on_existing = module.params.get("on_existing")
    no_mod = False
    if module.params.get("state") == "present":
        if records and mismatch:
            # Mismatch: user wants to overwrite?
            if on_existing == "replace":
                to_delete.extend(mismatch_records)
            elif on_existing == "keep_and_fail":
                module.fail_json(
                    msg="Record already exists with different value. Set on_existing=replace to replace it"
                )
            elif on_existing == "keep_and_warn":
                module.warn(
                    "Record already exists with different value. Set on_existing=replace to replace it"
                )
                no_mod = True
            else:  # on_existing == 'keep'
                no_mod = True
        if no_mod:
            after = before[:]  # type: ignore
        else:
            for target in values:
                if to_delete:
                    # If there's a record to delete, change it to new record
                    record = to_delete.pop()
                    to_change.append(record)
                    new_record: DNSRecord[RecordIDT | None] = record  # type: ignore
                else:
                    # Otherwise create new record
                    new_record = DNSRecord(
                        record_id=None, record_type=type_in, target=target
                    )
                    to_create.append(new_record)
                new_record.prefix = prefix
                new_record.type = type_in
                new_record.ttl = ttl_in
                new_record.target = target
                after.append(new_record)
    if module.params.get("state") == "absent":
        if mismatch:
            # Mismatch: user wants to overwrite?
            if on_existing == "replace":
                no_mod = False
            elif on_existing == "keep_and_fail":
                module.fail_json(
                    msg="Record already exists with different value. Set on_existing=replace to remove it"
                )
            elif on_existing == "keep_and_warn":
                module.warn(
                    "Record already exists with different value. Set on_existing=replace to remove it"
                )
                no_mod = True
            else:  # on_existing == 'keep'
                no_mod = True
        if no_mod:
            after = before[:]  # type: ignore
        else:
            to_delete.extend(records)
            after = []

    # Compose result
    result = {
        "changed": False,
        "zone_id": zone_id,
    }

    # Determine whether there's something to do
    if to_create or to_delete or to_change:
        # Actually do something
        records_to_delete = record_converter.clone_multiple_to_api(to_delete)
        records_to_change = record_converter.clone_multiple_to_api(to_change)
        records_to_create = record_converter.clone_multiple_to_api(to_create)
        result["changed"] = True
        if not module.check_mode:
            dummy, errors, dummy2 = bulk_apply_changes(
                api,
                zone_id=zone_id,
                records_to_delete=records_to_delete,
                records_to_change=records_to_change,
                records_to_create=records_to_create,
                provider_information=provider_information,
                options=option_provider,
            )
            if errors:
                if len(errors) == 1:
                    raise errors[0]
                module.fail_json(
                    msg=f"Errors: {'; '.join([str(e) for e in errors])}",
                    errors=[str(e) for e in errors],
                )

    # Include diff information
    if module._diff:
        result["diff"] = {
            "before": (
                format_records_for_output(
                    sorted(before, key=lambda record: record.target),
                    record_in,
                    prefix,
                    record_converter=record_converter,
                )
                if before
                else {}
            ),
            "after": (
                format_records_for_output(
                    sorted(after, key=lambda record: record.target),
                    record_in,
                    prefix,
                    record_converter=record_converter,
                )
                if after
                else {}
            ),
        }

    module.exit_json(**result)


def _run_module_record_set_api(
    option_provider,
    module: AnsibleModule,
    provider_information: ProviderInformation,
    record_converter: RecordConverter,
    record_in: str,
    prefix_in: str | None,
    type_in: str,
    api: ZoneRecordSetAPI[ZoneIDT, RecordSetIDT, RecordIDT],
) -> t.NoReturn:
    # Get zone information
    if module.params.get("zone_name") is not None:
        zone_in = normalize_dns_name(module.params.get("zone_name"))
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone = api.get_zone_with_record_sets_by_name(
            zone_in, prefix=prefix, record_type=type_in
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_id = zone.zone.id
        record_sets = zone.record_sets
    elif record_in is not None:
        zone = api.get_zone_with_record_sets_by_id(
            module.params.get("zone_id"),
            record_type=type_in,
            prefix=(
                provider_information.normalize_prefix(prefix_in)
                if prefix_in is not None
                else NOT_PROVIDED
            ),
        )
        if zone is None:
            module.fail_json(msg="Zone not found")
        zone_in = normalize_dns_name(zone.zone.name)
        record_in, prefix = get_prefix(
            normalized_zone=zone_in,
            normalized_record=record_in,
            prefix=prefix_in,
            provider_information=provider_information,
        )
        zone_id = zone.zone.id
        record_sets = zone.record_sets
    else:
        zone_id = module.params.get("zone_id")
        prefix = provider_information.normalize_prefix(prefix_in)
        record_sets = api.get_zone_record_sets(
            zone_id,
            record_type=type_in,
            prefix=prefix,
        )
        if record_sets is None:
            module.fail_json(msg="Zone not found")
        zone_in = None
        record_in = None

    # Find matching records
    record_sets = filter_record_sets(record_sets, prefix=prefix)
    if len(record_sets) > 1:
        module.fail_json(
            msg=f"Internal error: should have at most one record set, but got {len(record_sets)}"
        )  # pragma: no cover
    record_set = None
    if record_sets:
        record_set = record_sets[0]
        record_converter.process_set_from_api(record_set)

    # Parse records
    values = []
    value_in = module.params.get("value") or []
    value_in = record_converter.process_values_from_user(type_in, value_in)
    values = value_in[:]

    # Compare records
    ttl_in = module.params.get("ttl")
    mismatch_records = []
    keep_records = []
    if record_set:
        mismatch_ttl = record_set.ttl != ttl_in
        for record in record_set.records:
            val = record.target
            if val in values:
                values.remove(val)
                keep_records.append(record)
            else:
                mismatch_records.append(record)
    else:
        mismatch_ttl = False
        mismatch_records = []

    before = record_set.clone() if record_set else None
    after: IDNSRecordSet[RecordSetIDT | None, RecordIDT | None] | None = before

    # Compose result
    result = {
        "changed": False,
        "zone_id": zone_id,
    }

    # Proceed
    on_existing = module.params.get("on_existing")
    no_mod = False
    if module.params.get("state") == "present":
        mismatch_values = bool(mismatch_records or values)
        if (mismatch_values or mismatch_ttl) and record_set:
            # Mismatch: user wants to overwrite?
            if on_existing == "replace":
                pass
            elif on_existing == "keep_and_fail":
                module.fail_json(
                    msg="Record already exists with different value. Set on_existing=replace to replace it"
                )
            elif on_existing == "keep_and_warn":
                module.warn(
                    "Record already exists with different value. Set on_existing=replace to replace it"
                )
                no_mod = True
            else:  # on_existing == 'keep'
                no_mod = True
        if not no_mod:
            new_after: DNSRecordSet[RecordSetIDT | None, RecordIDT | None] = (
                DNSRecordSet(record_set_id=None, record_type=type_in)
            )
            after = new_after
            new_after.ttl = ttl_in
            new_after.prefix = prefix
            new_after.records.extend(keep_records)
            if record_set:
                new_after.id = record_set.id
                new_after.extra = record_set.extra.copy()
            for value in values:
                rec = DNSRecord(record_id=None, record_type=type_in, target=value)
                rec.ttl = ttl_in
                rec.prefix = prefix
                new_after.records.append(rec)
            if not new_after.records:
                after = None
                if record_set:
                    result["changed"] = True
                    if not module.check_mode:
                        api.delete_record_set(zone_id, record_set)
            elif mismatch_ttl or mismatch_values:
                result["changed"] = True
                if not module.check_mode:
                    new_record_set = record_converter.clone_set_to_api(new_after)
                    if record_set:
                        new_record_set.id = record_set.id
                        new_after = api.update_record_set(
                            zone_id,
                            new_record_set,  # type: ignore
                            updated_records=mismatch_values,
                            updated_ttl=mismatch_ttl,
                        )
                    else:
                        new_after = api.add_record_set(zone_id, new_record_set)  # type: ignore
                    after = record_converter.process_set_from_api(new_after)
    if module.params.get("state") == "absent":
        mismatch_values = bool(mismatch_records)
        if (mismatch_values or mismatch_ttl) and record_set:
            # Mismatch: user wants to overwrite?
            if on_existing == "replace":
                no_mod = False
            elif on_existing == "keep_and_fail":
                module.fail_json(
                    msg="Record already exists with different value. Set on_existing=replace to remove it"
                )
            elif on_existing == "keep_and_warn":
                module.warn(
                    "Record already exists with different value. Set on_existing=replace to remove it"
                )
                no_mod = True
            else:  # on_existing == 'keep'
                no_mod = True
        if not no_mod:
            after = None
            if record_set:
                result["changed"] = True
                if not module.check_mode:
                    api.delete_record_set(zone_id, record_set)

    # Include diff information
    if module._diff:
        result["diff"] = {
            "before": (
                format_record_set_for_output(
                    before, record_in, prefix, record_converter=record_converter
                )
                if before
                else {}
            ),
            "after": (
                format_record_set_for_output(
                    after, record_in, prefix, record_converter=record_converter
                )
                if after
                else {}
            ),
        }

    module.exit_json(**result)


def run_module(
    module: AnsibleModule,
    create_api: t.Callable[[], ZoneRecordAPI | ZoneRecordSetAPI],
    provider_information: ProviderInformation,
) -> t.NoReturn:
    option_provider = ModuleOptionProvider(module)
    record_converter = RecordConverter(provider_information, option_provider)
    record_converter.emit_deprecations(module.deprecate)

    record_in = normalize_dns_name(module.params.get("record"))
    prefix_in = module.params.get("prefix")
    type_in = module.params.get("type")
    if type_in and type_in not in provider_information.get_supported_record_types():
        module.fail_json(msg=f"Invalid record type {type_in}")

    try:
        # Create API
        api = create_api()

        if isinstance(api, ZoneRecordAPI):
            _run_module_record_api(
                option_provider,
                module,
                provider_information,
                record_converter,
                record_in,
                prefix_in,
                type_in,
                api,
            )
        else:
            _run_module_record_set_api(
                option_provider,
                module,
                provider_information,
                record_converter,
                record_in,
                prefix_in,
                type_in,
                api,
            )

    except DNSConversionError as e:
        module.fail_json(
            msg=f"Error while converting DNS values: {e.error_message}",
            error=e.error_message,
            exception=traceback.format_exc(),
        )
    except DNSAPIAuthenticationError as e:
        module.fail_json(
            msg=f"Cannot authenticate: {e}",
            error=to_text(e),
            exception=traceback.format_exc(),
        )
    except DNSAPIError as e:
        module.fail_json(
            msg=f"Error: {e}",
            error=to_text(e),
            exception=traceback.format_exc(),
        )



#!/usr/bin/python
#
# Copyright (c) 2020 Haiyuan Zhang, <haiyzhan@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
module: azure_rm_adserviceprincipal_info

version_added: "0.2.0"

short_description: Get Azure Active Directory service principal info

description:
    - Get Azure Active Directory service principal info.

options:
    app_id:
        description:
            - The application ID.
        type: str
    object_id:
        description:
            - It's service principal's object ID.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    haiyuan_zhang (@haiyuazhang)
    Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: get ad sp info
  azure_rm_adserviceprincipal_info:
    app_id: "{{ app_id }}"
- name: get all service principals
  azure_rm_adserviceprincipal_info:
'''

RETURN = '''
service_principals:
    description:
        - A list of service principals in the tenant. If app_id or object_id is set, the maximum length
          of this list should be one.
    type: list
    elements: dict
    returned: always
    contains:
        account_enabled:
            description:
                - Whether the service principal is enabled.
            type: bool
            returned: always
            sample: true
        app_display_name:
            description:
                - Object's display name or its prefix.
            type: str
            returned: always
            sample: sp
        app_id:
            description:
                - The application ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        app_role_assignment_required:
            description:
                - Whether the Role of the Service Principal is set.
            type: bool
            returned: always
            sample: false
        notes:
            description:
                - The notes of the Service Principal.
            type: str
            returned: always
        object_id:
            description:
                - It's service principal's object ID.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        service_principal_type:
            description:
                - The type of the service principal. Application or ManagedIdentity.
            type: str
            returned: always
            sample: application


'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase

try:
    import asyncio
    from msgraph.generated.service_principals.service_principals_request_builder import ServicePrincipalsRequestBuilder
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMADServicePrincipalInfo(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            app_id=dict(type='str'),
            object_id=dict(type='str'),
        )

        self.app_id = None
        self.object_id = None
        self.results = dict(changed=False)

        super(AzureRMADServicePrincipalInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                            supports_check_mode=True,
                                                            supports_tags=False,
                                                            is_ad_resource=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self._client = self.get_msgraph_client()

        service_principals = []

        try:
            if self.object_id is None:
                service_principals = asyncio.get_event_loop().run_until_complete(self.get_service_principals())
            else:
                service_principals = [asyncio.get_event_loop().run_until_complete(self.get_service_principal())]

            self.results['service_principals'] = [self.to_dict(sp) for sp in service_principals]
        except Exception as ge:
            self.fail("failed to get service principal info {0}".format(str(ge)))

        return self.results

    def to_dict(self, object):
        return dict(
            app_id=object.app_id,
            object_id=object.id,
            app_display_name=object.display_name,
            app_role_assignment_required=object.app_role_assignment_required,
            notes=object.notes,
            account_enabled=object.account_enabled,
            service_principal_type=object.service_principal_type
        )

    async def get_service_principal(self):
        return await self._client.service_principals.by_service_principal_id(self.object_id).get()

    async def get_service_principals(self):
        kwargs = {}
        if self.app_id is not None:
            request_configuration = ServicePrincipalsRequestBuilder.ServicePrincipalsRequestBuilderGetRequestConfiguration(
                query_parameters=ServicePrincipalsRequestBuilder.ServicePrincipalsRequestBuilderGetQueryParameters(
                    filter="servicePrincipalNames/any(c:c eq '{0}')".format(self.app_id))
            )
            kwargs['request_configuration'] = request_configuration
        service_principals = []
        response = await self._client.service_principals.get(**kwargs)
        if response:
            service_principals += response.value
        while response is not None and response.odata_next_link is not None:
            response = await self._client.service_principals.with_url(response.odata_next_link).get(**kwargs)
            if response:
                service_principals += response.value
        return service_principals


def main():
    AzureRMADServicePrincipalInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: restconf_config
author: Ganesh Nalawade (@ganeshrn)
short_description: Handles create, update, read and delete of configuration data on
  RESTCONF enabled devices.
description:
- RESTCONF is a standard mechanisms to allow web applications to configure and manage
  data. RESTCONF is a IETF standard and documented on RFC 8040.
- This module allows the user to configure data on RESTCONF enabled devices.
version_added: 1.0.0
options:
  path:
    description:
    - URI being used to execute API calls.
    required: true
    type: str
  content:
    description:
    - The configuration data in format as specififed in C(format) option. Required
      unless C(method) is I(delete).
    type: str
  method:
    description:
    - The RESTCONF method to manage the configuration change on device. The value
      I(post) is used to create a data resource or invoke an operation resource, I(put)
      is used to replace the target data resource, I(patch) is used to modify the
      target resource, and I(delete) is used to delete the target resource.
    required: false
    type: str
    default: post
    choices:
    - post
    - put
    - patch
    - delete
  format:
    description:
    - The format of the configuration provided as value of C(content). Accepted values
      are I(xml) and I(json) and the given configuration format should be supported
      by remote RESTCONF server.
    type: str
    default: json
    choices:
    - json
    - xml
notes:
- This module requires the RESTCONF system service be enabled on the remote device
  being managed.
- This module is supported with I(ansible_connection) value of I(ansible.netcommon.httpapi) and
  I(ansible_network_os) value of I(ansible.netcommon.restconf).
- This module is tested against Cisco IOSXE 16.12.02 version.
"""

EXAMPLES = """
- name: create l3vpn services
  ansible.netcommon.restconf_config:
    path: /config/ietf-l3vpn-svc:l3vpn-svc/vpn-services
    content: |
      {
        "vpn-service":[
                        {
                          "vpn-id": "red_vpn2",
                          "customer-name": "blue",
                          "vpn-service-topology": "ietf-l3vpn-svc:any-to-any"
                        },
                        {
                          "vpn-id": "blue_vpn1",
                          "customer-name": "red",
                          "vpn-service-topology": "ietf-l3vpn-svc:any-to-any"
                        }
                      ]
       }
"""

RETURN = """
candidate:
  description: The configuration sent to the device.
  returned: When the method is not delete
  type: dict
  sample: |
        {
            "vpn-service": [
                {
                    "customer-name": "red",
                    "vpn-id": "blue_vpn1",
                    "vpn-service-topology": "ietf-l3vpn-svc:any-to-any"
                }
            ]
        }
running:
  description: The current running configuration on the device.
  returned: When the method is not delete
  type: dict
  sample: |
        {
            "vpn-service": [
                {
                  "vpn-id": "red_vpn2",
                  "customer-name": "blue",
                  "vpn-service-topology": "ietf-l3vpn-svc:any-to-any"
                },
                {
                  "vpn-id": "blue_vpn1",
                  "customer-name": "red",
                  "vpn-service-topology": "ietf-l3vpn-svc:any-to-any"
                }
            ]
        }

"""

import json

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_text
from ansible.module_utils.connection import ConnectionError

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.restconf import restconf


string_types = (str,)


def main():
    """entry point for module execution"""
    argument_spec = dict(
        path=dict(required=True),
        content=dict(),
        method=dict(choices=["post", "put", "patch", "delete"], default="post"),
        format=dict(choices=["json", "xml"], default="json"),
    )
    required_if = [
        ["method", "post", ["content"]],
        ["method", "put", ["content"]],
        ["method", "patch", ["content"]],
    ]

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    path = module.params["path"]
    candidate = module.params["content"]
    method = module.params["method"]
    format = module.params["format"]

    if isinstance(candidate, string_types):
        candidate = json.loads(candidate)

    warnings = list()
    result = {"changed": False, "warnings": warnings}

    running = None
    commit = not module.check_mode
    try:
        running = restconf.get(module, path, output=format)
    except ConnectionError as exc:
        if exc.code == 404:
            running = None
        else:
            module.fail_json(msg=to_text(exc), code=exc.code)

    try:
        if method == "delete":
            if running:
                if commit:
                    restconf.edit_config(module, path=path, method="DELETE")
                result["changed"] = True
            else:
                warnings.append("delete not executed as resource '%s' does not exist" % path)
        else:
            if running:
                diff = dict_diff(running, candidate)
                result["candidate"] = candidate
                result["running"] = running
            else:
                diff = candidate

            if diff:
                if module._diff:
                    result["diff"] = {
                        "prepared": diff,
                        "before": candidate,
                        "after": running,
                    }

                if commit:
                    restconf.edit_config(
                        module,
                        path=path,
                        content=diff,
                        method=method.upper(),
                        format=format,
                    )
                result["changed"] = True

    except ConnectionError as exc:
        module.fail_json(msg=str(exc), code=exc.code)

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_wwn_pool_info
short_description: Gather information about WWN Pool in Cisco Intersight
description:
  - Gather information about WWNN/WWPN Pool in L(Cisco Intersight,https://intersight.com).
  - Information can be filtered using Organization, Name and PoolPurpose. if none is passed all information regarding existing
    WWN pools will be fetched.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization that will have information gathered from.
    type: str
  name:
    description:
      - The name of the WWNN/WWPN Pool that will have information gathered from.
    type: str
  pool_purpose:
    description:
      - The pool type WWNN or WWPN.
      - WWNN and WWPN pools can have the same name. if none is passed all information regardless of pool purpose will be fetched.
    type: str
author:
  - Shahar Golshani (@sgolshan)
'''

EXAMPLES = r'''
- name: Fetch wwn pool information by name
  cisco.intersight.intersight_wwn_pool_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: wwn_pool_1

- name: Fetch all wwn pool information
  cisco.intersight.intersight_wwn_pool_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": [
      {
          "Name": "wwn_pool_1",
          "ObjectType": "fcpool.Pool",
          "Tags": [
              {
                  "Key": "Site",
                  "Value": "tag1"
              }
          ],
          "IdBlocks": [
              {
                  "ClassId": "fcpool.Block",
                  "From": "20:00:00:25:B5:00:00:00",
                  "ObjectType": "fcpool.Block",
                  "Size": 100,
                  "To": "20:00:00:25:B5:00:00:63"
              }
          ],
          "PoolPurpose": "WWPN"
      }
    ]
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str'),
        pool_purpose=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to fetch info
    resource_path = '/fcpool/Pools'

    query_params = intersight.set_query_params(filter_key="PoolPurpose", filter_value=intersight.module.params['pool_purpose'])
    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2022 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_sqlmanagedinstance
version_added: '1.14.0'
short_description: Manage SQL managed instances
description:
    - Create, update, or delete SQL managed instances.

options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
        required: True
    name:
        description:
            - The name of the sql managed instance.
        type: str
        required: True
    location:
        description:
            - The location of the sql managed instance.
        type: str
    sku:
        description:
            - An ARM Resource SKU.
        type: dict
        suboptions:
            name:
                description:
                    - The name of the SKU, typically, a letter add Number code.
                type: str
            tier:
                description:
                    - The tier or edition of the particular SKU.
                type: str
            size:
                description:
                    - Size of the particular SKU.
                type: str
            family:
                description:
                    - If the service has different generations of hardware, for the same SKU, then that can be captured here.
                type: str
            capacity:
                description:
                    - The capacity of the managed instance in integer number of vcores.
                type: str
    administrators:
        description:
            - The Azure Active Directory administrator of the server.
        type: str
    managed_instance_create_mode:
        description:
            - Specifies the mode of database creation.
        type: str
    administrator_login:
        description:
            - Administrator username for the managed instance.
            - Can only be specified when the managed instance is being created (and is required for creation).
        type: str
    administrator_login_password:
        description:
            - The administrator login password (required for managed instance creation).
        type: str
    subnet_id:
        description:
            - Subnet resource ID for the managed instance.
        type: str
    license_type:
        description:
            - The license type.
            - Possible values are C(LicenseIncluded) and C(BasePrice).
            - Discounted AHB price for bringing your own SQL licenses.
            - Regular price inclusive of a new SQL license.
        type: str
        choices:
            - LicenseIncluded
            - BasePrice
    v_cores:
        description:
            - The number of vCores.
        type: int
        choices:
            - 8
            - 16
            - 24
            - 32
            - 40
            - 64
            - 80
    storage_size_in_gb:
        description:
            - Storage size in GB.
            - Minimum value is C(32). Maximum value is C(8192).
            - Increments of 32 GB allowed only.
        type: int
    collation:
        description:
            - Collation of the managed instance.
        type: str
    dns_zone:
        description:
            - The Dns Zone that the managed instance is in.
        type: str
    dns_zone_partner:
        description:
            - The resource ID of another managed instance whose DNS zone this managed instance will share after creation.
        type: str
    public_data_endpoint_enabled:
        description:
            - Whether or not the public data endpoint is enabled.
        type: bool
    source_managed_instance_id:
        description:
            - The resource identifier of the source managed instance associated with create operation of this instance.
        type: str
    restore_point_in_time:
        description:
            -  Specifies the point in time (ISO8601 format) of the source database that will be restored to create the new database.
        type: str
    proxy_override:
        description:
            - Connection type used for connecting to the instance.
        type: str
        choices:
            - Proxy
            - Redirect
            - Default
    timezone_id:
        description:
            - ID of the timezone.
            - Allowed values are timezones supported by Windows.
            - Windows keeps details on supported timezones.
        type: str
    instance_pool_id:
        description:
            - The ID of the instance pool this managed server belongs to.
        type: str
    private_endpoint_connections:
        description:
            - List of private endpoint connections on a managed instance.
        type: list
        elements: str
    maintenance_configuration_id:
        description:
            - Specifies maintenance configuration ID to apply to this managed instance.
        type: str
    minimal_tls_version:
        description:
            - Minimal TLS version. Allowed values C(None), C(1.0), C(1.1), C(1.2).
        type: str
        choices:
            - 'None'
            - '1.0'
            - '1.1'
            - '1.2'
    storage_account_type:
        description:
            - The storage account type used to store backups for this instance.
        type: str
    zone_redundant:
        description:
            - Whether or not the multi-az is enabled.
        type: bool
    primary_user_assigned_identity_id:
        description:
            - The resource id of a user assigned identity to be used by default.
        type: str
    key_id:
        description:
            - A CMK URI of the key to use for encryption.
        type: str
    state:
        description:
            - State of the sql managed instance.
            - Use C(present) to create or update a automation runbook and use C(absent) to delete.
        type: str
        default: present
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
    - azure.azcollection.azure_identity_multiple

author:
    - xuzhang3 (@xuzhang3)
    - Fred Sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Create sql managed instance
  azure_rm_sqlmanagedinstance:
    resource_group: "{{ resource_group }}"
    name: testmanagedinstance
    subnet_id: subnet_id
    sku:
      name: GP_Gen5
      tier: GeneralPurpose
      family: Gen5
      capacity: 5
    identity:
      type: SystemAssigned
    administrator_login: azureuser
    administrator_login_password: "{{ password }}"
    storage_size_in_gb: 256
    v_cores: 8

- name: Delete sql managed instance
  azure_rm_sqlmanagedinstance:
    resource_group: "{{ resource_group }}"
    name: testmanagedinstance
    state: absent
'''

RETURN = '''
sql_managed_instance:
    description:
        - A list of dictionaries containing facts for SQL Managed Instance.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: "/subscription/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Sql/managedInstances/fredsqlinstance"
        name:
            description:
                - SQL managed instance name.
            returned: always
            type: str
            sample: testmanagedinstance
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: { 'taga':'aaa', 'tagb':'bbb' }
        identity:
            description:
                - Azure Active Directory identity configuration for a resource.
            returned: always
            type: complex
            contains:
                principal_id:
                    description:
                        - The Azure Active Directory principal ID.
                    type: str
                    returned: always
                    sample: 895c-xxx-xxxbe
                tenant_id:
                    description:
                        - The Azure Active Directory tenant ID.
                    type: str
                    returned: always
                    sample: 72fxxxxx-xxxx-xxxx-xxxx-xxxxxx11db47
                type:
                    description:
                        - The identity type.
                    type: str
                    returned: always
                    sample: SystemAssigned
                user_assigned_identities:
                    description:
                        - The resource ids of the user assigned identities to use.
                    type: str
                    returned: always
                    sample: null
        sku:
            description:
                - An ARM Resource SKU.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - The name of the SKU.
                    returned: always
                    type: str
                    sample: BC_Gen4_2
                tier:
                    description:
                        - The SKU tier.
                    returned: always
                    type: str
                    sample: BusinessCritical
                capacity:
                    description:
                        - The SKU capacity.
                    returned: always
                    type: int
                    sample: 2
                family:
                    description:
                        - If the service has different generations of hardware, for the same SKU, then that can be captured here.
                    type: str
                    returned: always
                    sample: Gen5
                size:
                    description:
                        - Size of the particular SKU.
                    type: str
                    returned: always
                    sample: null
        collation:
            description:
                - The collation of the SQL managed instance.
            returned: always
            type: str
            sample: SQL_Latin1_General_CP1_CI_AS
        administrator_login:
            description:
                - Administrator username for the managed instance.
            type: str
            returned: always
            sample: azureuser
        administrators:
            description:
                - The Azure Active Directory administrator of the server.
            type: str
            returned: always
            sample: null
        dns_zone:
            description:
                -The Dns Zone that the managed instance is in.
            type: str
            returned: always
            sample: 8a23abba54cd
        dns_zone_partner:
            description:
                - The resource ID of another managed instance whose DNS zone this managed instance will share after creation.
            type: str
            returned: always
            sample: null
        fully_qualified_domain_name:
            description:
                - The fully qualified domain name of the managed instance.
            type: str
            returned: always
            sample: fredsqlinstance.8a23abba54cd.database.windows.net
        instance_pool_id:
            description:
                - The ID of the instance pool this managed server belongs to.
            type: str
            returned: always
            sample: null
        key_id:
            description:
                - A CMK URI of the key to use for encryption.
            type: str
            returned: always
            sample: null
        license_type:
            description:
                - The license type.
            type: str
            returned: always
            sample: LicenseIncluded
        maintenance_configuration_id:
            description:
                - Specifies maintenance configuration ID to apply to this managed instance.
            type: str
            returned: always
            sample: /subscriptions/xxx-xxxx/providers/Microsoft.Maintenance/publicMaintenanceConfigurations/SQL_Default
        managed_instance_create_mode:
            description:
                - Specifies the mode of database creation.
            type: str
            returned: always
            sample: null
        minimal_tls_version:
            description:
                - Minimal TLS version. Allowed values 'None', '1.0', '1.1', '1.2'.
            type: str
            returned: always
            sample: 1.2
        primary_user_assigned_identity_id:
            description:
                - The resource id of a user assigned identity to be used by default.
            type: str
            returned: always
            sample: null
        private_endpoint_connections:
            description:
                - List of private endpoint connections on a managed instance.
            type: list
            returned: always
            sample: []
        provisioning_state:
            description:
                - The Status of the SQL managed instance.
            type: str
            returned: always
            sample: Successed
        proxy_override:
            description:
                - Connection type used for connecting to the instance.
            type: str
            returned: always
            sample: Proxy
        public_data_endpoint_enabled:
            description:
                - Whether or not the public data endpoint is enabled.
            type: bool
            returned: always
            sample: false
        restore_point_in_time:
            description:
                - Specifies the point in time (ISO8601 format) of the source database that will be restored to create the new database.
            type: str
            returned: always
            sample: null
        source_managed_instance_id:
            description:
                - The resource identifier of the source managed instance associated with create operation of this instance.
            type: str
            returned: always
            sample: null
        state:
            description:
                - The state of the managed instance.
            type: str
            returned: always
            sample: Ready
        storage_account_type:
            description:
                - The storage account type used to store backups for this instance.
            type: str
            returned: always
            sample: GRS
        storage_size_in_gb:
            description:
                - Storage size in GB. Minimum value 32. Maximum value 8192.
            type: int
            returned: always
            sample: 256
        subnet_id:
            description:
                - Subnet resource ID for the managed instance.
            type: str
            returned: always
            sample: /subscriptions/xxx-xxxx/resourceGroups/testRG/providers/Microsoft.Network/virtualNetworks/vnet-smi/subnets/sqi_sub
        timezone_id:
            description:
                -  Id of the timezone. Allowed values are timezones supported by Windows.
            type: str
            returned: always
            sample: UTC
        type:
            description:
                - The SQL managed instance type.
            type: str
            returned: always
            sample: "Microsoft.Sql/managedInstances"
        v_cores:
            description:
                - The number of vCores. Allowed values 8, 16, 24, 32, 40, 64, 80.
            type: int
            returned: always
            sample: 8
        zone_redundant:
            description:
                - Whether or not the multi-az is enabled.
            type: bool
            returned: always
            sample: false
'''

# from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.sql.models import (ResourceIdentity, UserIdentity)
    from azure.core.polling import LROPoller
except ImportError:
    pass


sku_spec = dict(
    name=dict(type='str'),
    tier=dict(type='str'),
    size=dict(type='str'),
    family=dict(type='str'),
    capacity=dict(type='str')
)


user_assigned_identities_spec = dict(
    id=dict(
        type='list',
        default=[],
        elements='str'
    )
)


# class AzureRMSqlManagedInstance(AzureRMModuleBase):
class AzureRMSqlManagedInstance(AzureRMModuleBaseExt):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            location=dict(
                type='str'
            ),
            subnet_id=dict(
                type='str'
            ),
            identity=dict(
                type='dict',
                options=self.managed_identity_multiple_spec
            ),
            sku=dict(
                type='dict',
                options=sku_spec
            ),
            managed_instance_create_mode=dict(
                type='str'
            ),
            administrator_login=dict(
                type='str',
            ),
            administrator_login_password=dict(
                type='str',
                no_log=True,
            ),
            license_type=dict(
                type='str',
                choices=['LicenseIncluded', 'BasePrice']
            ),
            v_cores=dict(
                type='int',
                choices=[8, 16, 24, 32, 40, 64, 80]
            ),
            storage_size_in_gb=dict(
                type='int'
            ),
            collation=dict(
                type='str'
            ),
            dns_zone=dict(
                type='str'
            ),
            dns_zone_partner=dict(
                type='str'
            ),
            public_data_endpoint_enabled=dict(
                type='bool'
            ),
            source_managed_instance_id=dict(
                type='str'
            ),
            restore_point_in_time=dict(
                type='str'
            ),
            proxy_override=dict(
                type='str',
                choices=['Proxy', 'Redirect', 'Default']
            ),
            timezone_id=dict(
                type='str'
            ),
            instance_pool_id=dict(
                type='str'
            ),
            maintenance_configuration_id=dict(
                type='str'
            ),
            private_endpoint_connections=dict(
                type='list',
                elements='str'
            ),
            minimal_tls_version=dict(
                type='str',
                choices=['None', '1.0', '1.1', '1.2']
            ),
            storage_account_type=dict(
                type='str'
            ),
            zone_redundant=dict(
                type='bool'
            ),
            primary_user_assigned_identity_id=dict(
                type='str'
            ),
            key_id=dict(
                type='str'
            ),
            administrators=dict(
                type='str'
            ),
            state=dict(
                type='str',
                choices=['present', 'absent'],
                default='present'
            )
        )
        # store the results of the module operation
        self.results = dict(changed=False)
        self.resource_group = None
        self.name = None
        self.location = None
        self.state = None
        self.identity = None
        self.body = dict()
        self._managed_identity = None

        super(AzureRMSqlManagedInstance, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {"identity": ResourceIdentity,
                                      "user_assigned": UserIdentity
                                      }
        return self._managed_identity

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)

        if not self.location:
            resource_group = self.get_resource_group(self.resource_group)
            self.location = resource_group.location
        self.body['location'] = self.location

        sql_managed_instance = self.get()

        update_identity, identity = self.update_managed_identity(new_identity=self.identity or {},
                                                                 curr_identity=sql_managed_instance and
                                                                 sql_managed_instance.get('identity', {}) or {})
        if update_identity:
            self.body["identity"] = identity.as_dict()

        changed = False
        if self.state == 'present':
            if sql_managed_instance:
                modifiers = {}
                self.create_compare_modifiers(self.module_arg_spec, '', modifiers)
                self.results['modifiers'] = modifiers
                self.results['compare'] = []
                if not self.default_compare(modifiers, self.body, sql_managed_instance, '', self.results):
                    changed = True

                if changed or update_identity:
                    if not self.check_mode:
                        # sql_managed_instance = self.update_sql_managed_instance(self.body)
                        sql_managed_instance = self.create_or_update(self.body)
            else:
                changed = True
                if not self.check_mode:
                    sql_managed_instance = self.create_or_update(self.body)

        else:
            changed = True
            if not self.check_mode:
                sql_managed_instance = self.delete_sql_managed_instance()

        self.results['changed'] = changed
        self.results['state'] = sql_managed_instance
        return self.results

    def get(self):
        try:
            response = self.sql_client.managed_instances.get(self.resource_group, self.name)
            return self.to_dict(response)
        except ResourceNotFoundError:
            pass

    def update_sql_managed_instance(self, parameters):
        try:
            response = self.sql_client.managed_instances.begin_update(resource_group_name=self.resource_group,
                                                                      managed_instance_name=self.name,
                                                                      parameters=parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return self.to_dict(response)
        except Exception as exc:
            self.fail('Error when updating SQL managed instance {0}: {1}'.format(self.name, exc.message))

    def create_or_update(self, parameters):
        try:
            response = self.sql_client.managed_instances.begin_create_or_update(resource_group_name=self.resource_group,
                                                                                managed_instance_name=self.name,
                                                                                parameters=parameters)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)
            return self.to_dict(response)
        except Exception as exc:
            self.fail('Error when creating SQL managed instance {0}: {1}'.format(self.name, exc))

    def delete_sql_managed_instance(self):
        try:
            response = self.sql_client.managed_instances.begin_delete(self.resource_group, self.name)
            if isinstance(response, LROPoller):
                self.get_poller_result(response)

        except Exception as exc:
            self.fail('Error when deleting SQL managed instance {0}: {1}'.format(self.name, exc))

    def to_dict(self, item):
        if not item:
            return None
        d = item.as_dict()
        d = {
            'resource_group': self.resource_group,
            'id': d.get('id', None),
            'name': d.get('name', None),
            'location': d.get('location', None),
            'type': d.get('type', None),
            'tags': d.get('tags', None),
            'identity': {
                'user_assigned_identities': d.get('identity', {}).get('user_assigned_identities', None),
                'principal_id': d.get('identity', {}).get('principal_id', None),
                'type': d.get('identity', {}).get('type', None),
                'tenant_id': d.get('identity', {}).get('tenant_id', None)
            },
            'sku': {
                'name': d.get('sku', {}).get('name', None),
                'size': d.get('sku', {}).get('size', None),
                'family': d.get('sku', {}).get('family', None),
                'tier': d.get('sku', {}).get('tier', None),
                'capacity': d.get('sku', {}).get('capacity', None)
            },
            'provisioning_state': d.get('provisioning_state', None),
            'managed_instance_create_mode': d.get('managed_instance_create_mode', None),
            'fully_qualified_domain_name': d.get('fully_qualified_domain_name', None),
            'administrator_login': d.get('administrator_login', None),
            'subnet_id': d.get('subnet_id', None),
            'state': d.get('state', None),
            'license_type': d.get('license_type', None),
            'v_cores': d.get('v_cores', None),
            'storage_size_in_gb': d.get('storage_size_in_gb', None),
            'collation': d.get('collation', None),
            'dns_zone': d.get('dns_zone', None),
            'dns_zone_partner': d.get('dns_zone_partner', None),
            'public_data_endpoint_enabled': d.get('public_data_endpoint_enabled', None),
            'source_managed_instance_id': d.get('source_managed_instance_id', None),
            'restore_point_in_time': d.get('restore_point_in_time', None),
            'proxy_override': d.get('proxy_override', None),
            'timezone_id': d.get('timezone_id', None),
            'instance_pool_id': d.get('instance_pool_id', None),
            'maintenance_configuration_id': d.get('maintenance_configuration_id', None),
            'private_endpoint_connections': d.get('private_endpoint_connections', None),
            'minimal_tls_version': d.get('minimal_tls_version', None),
            'storage_account_type': d.get('storage_account_type', None),
            'zone_redundant': d.get('zone_redundant', None),
            'primary_user_assigned_identity_id': d.get('primary_user_assigned_identity_id', None),
            'key_id': d.get('key_id', None),
            'administrators': d.get('administrators', None)
        }
        return d


def main():
    AzureRMSqlManagedInstance()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2019, INSPQ <philippe.gauthier@inspq.qc.ca>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_authentication

short_description: Configure authentication in Keycloak

description:
  - This module actually can only make a copy of an existing authentication flow, add an execution to it and configure it.
  - It can also delete the flow.
version_added: "3.3.0"

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  action_group:
    version_added: 10.2.0

options:
  realm:
    description:
      - The name of the realm in which is the authentication.
    required: true
    type: str
  alias:
    description:
      - Alias for the authentication flow.
    required: true
    type: str
  description:
    description:
      - Description of the flow.
    type: str
  providerId:
    description:
      - C(providerId) for the new flow when not copied from an existing flow.
    choices: ["basic-flow", "client-flow"]
    type: str
  copyFrom:
    description:
      - C(flowAlias) of the authentication flow to use for the copy.
    type: str
  authenticationExecutions:
    description:
      - Configuration structure for the executions.
    type: list
    elements: dict
    suboptions:
      providerId:
        description:
          - C(providerID) for the new flow when not copied from an existing flow.
        type: str
      displayName:
        description:
          - Name of the execution or subflow to create or update.
        type: str
      requirement:
        description:
          - Control status of the subflow or execution.
        choices: ["REQUIRED", "ALTERNATIVE", "DISABLED", "CONDITIONAL"]
        type: str
      flowAlias:
        description:
          - Alias of parent flow.
        type: str
      authenticationConfig:
        description:
          - Describe the config of the authentication.
        type: dict
      index:
        description:
          - Priority order of the execution.
        type: int
      subFlowType:
        description:
          - For new subflows, optionally specify the type.
          - Is only used at creation.
        choices: ["basic-flow", "form-flow"]
        default: "basic-flow"
        type: str
        version_added: 6.6.0
  state:
    description:
      - Control if the authentication flow must exists or not.
    choices: ["present", "absent"]
    default: present
    type: str
  force:
    type: bool
    default: false
    description:
      - If V(true), allows to remove the authentication flow and recreate it.
extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes

author:
  - Philippe Gauthier (@elfelip)
  - Gaëtan Daubresse (@Gaetan2907)
"""

EXAMPLES = r"""
- name: Create an authentication flow from first broker login and add an execution to it.
  community.general.keycloak_authentication:
    auth_keycloak_url: http://localhost:8080/auth
    auth_realm: master
    auth_username: admin
    auth_password: password
    realm: master
    alias: "Copy of first broker login"
    copyFrom: "first broker login"
    authenticationExecutions:
      - providerId: "test-execution1"
        requirement: "REQUIRED"
        authenticationConfig:
          alias: "test.execution1.property"
          config:
          test1.property: "value"
      - providerId: "test-execution2"
        requirement: "REQUIRED"
        authenticationConfig:
          alias: "test.execution2.property"
          config:
          test2.property: "value"
    state: present

- name: Re-create the authentication flow
  community.general.keycloak_authentication:
    auth_keycloak_url: http://localhost:8080/auth
    auth_realm: master
    auth_username: admin
    auth_password: password
    realm: master
    alias: "Copy of first broker login"
    copyFrom: "first broker login"
    authenticationExecutions:
      - providerId: "test-provisioning"
        requirement: "REQUIRED"
        authenticationConfig:
          alias: "test.provisioning.property"
          config:
          test.provisioning.property: "value"
    state: present
    force: true

- name: Create an authentication flow with subflow containing an execution.
  community.general.keycloak_authentication:
    auth_keycloak_url: http://localhost:8080/auth
    auth_realm: master
    auth_username: admin
    auth_password: password
    realm: master
    alias: "Copy of first broker login"
    copyFrom: "first broker login"
    authenticationExecutions:
      - providerId: "test-execution1"
        requirement: "REQUIRED"
      - displayName: "New Subflow"
        requirement: "REQUIRED"
      - providerId: "auth-cookie"
        requirement: "REQUIRED"
        flowAlias: "New Sublow"
    state: present

- name: Remove authentication.
  community.general.keycloak_authentication:
    auth_keycloak_url: http://localhost:8080/auth
    auth_realm: master
    auth_username: admin
    auth_password: password
    realm: master
    alias: "Copy of first broker login"
    state: absent
"""

RETURN = r"""
msg:
  description: Message as to what action was taken.
  returned: always
  type: str

end_state:
  description: Representation of the authentication after module execution.
  returned: on success
  type: dict
  sample:
    {
      "alias": "Copy of first broker login",
      "authenticationExecutions": [
        {
          "alias": "review profile config",
          "authenticationConfig": {
            "alias": "review profile config",
            "config": {
              "update.profile.on.first.login": "missing"
            },
            "id": "6f09e4fb-aad4-496a-b873-7fa9779df6d7"
          },
          "configurable": true,
          "displayName": "Review Profile",
          "id": "8f77dab8-2008-416f-989e-88b09ccf0b4c",
          "index": 0,
          "level": 0,
          "providerId": "idp-review-profile",
          "requirement": "REQUIRED",
          "requirementChoices": [
            "REQUIRED",
            "ALTERNATIVE",
            "DISABLED"
          ]
        }
      ],
      "builtIn": false,
      "description": "Actions taken after first broker login with identity provider account, which is not yet linked to any Keycloak account",
      "id": "bc228863-5887-4297-b898-4d988f8eaa5c",
      "providerId": "basic-flow",
      "topLevel": true
    }
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    get_token,
    is_struct_included,
    keycloak_argument_spec,
)


def find_exec_in_executions(searched_exec, executions):
    """
    Search if exec is contained in the executions.
    :param searched_exec: Execution to search for.
    :param executions: List of executions.
    :return: Index of the execution, -1 if not found..
    """
    for i, existing_exec in enumerate(executions, start=0):
        if (
            "providerId" in existing_exec
            and "providerId" in searched_exec
            and existing_exec["providerId"] == searched_exec["providerId"]
            or "displayName" in existing_exec
            and "displayName" in searched_exec
            and existing_exec["displayName"] == searched_exec["displayName"]
        ):
            return i
    return -1


def create_or_update_executions(kc, config, realm="master"):
    """
    Create or update executions for an authentication flow.
    :param kc: Keycloak API access.
    :param config: Representation of the authentication flow including its executions.
    :param realm: Realm
    :return: tuple (changed, dict(before, after)
        WHERE
        bool changed indicates if changes have been made
        dict(str, str) shows state before and after creation/update
    """
    try:
        changed = False
        after = ""
        before = ""
        execution = None
        if config.get("authenticationExecutions") is not None:
            # Get existing executions on the Keycloak server for this alias
            existing_executions = kc.get_executions_representation(config, realm=realm)
            for new_exec_index, new_exec in enumerate(config["authenticationExecutions"], start=0):
                if new_exec["index"] is not None:
                    new_exec_index = new_exec["index"]
                exec_found = False
                # Get flowalias parent if given
                if new_exec["flowAlias"] is not None:
                    flow_alias_parent = new_exec["flowAlias"]
                else:
                    flow_alias_parent = config["alias"]
                # Check if same providerId or displayName name between existing and new execution
                exec_index = find_exec_in_executions(new_exec, existing_executions)
                if exec_index != -1:
                    # Remove key that doesn't need to be compared with existing_exec
                    exclude_key = ["flowAlias", "subFlowType"]
                    for key in new_exec:
                        if new_exec[key] is None:
                            exclude_key.append(key)
                    # Compare the executions to see if it need changes
                    if (
                        not is_struct_included(new_exec, existing_executions[exec_index], exclude_key)
                        or exec_index != new_exec_index
                    ):
                        exec_found = True
                        if new_exec["index"] is None:
                            new_exec_index = exec_index
                        before += f"{existing_executions[exec_index]}\n"
                    execution = existing_executions[exec_index].copy()
                    # Remove exec from list in case 2 exec with same name
                    existing_executions[exec_index].clear()
                elif new_exec["providerId"] is not None:
                    kc.create_execution(new_exec, flowAlias=flow_alias_parent, realm=realm)
                    execution = kc.get_executions_representation(config, realm=realm)[exec_index]
                    exec_found = True
                    exec_index = new_exec_index
                    after += f"{new_exec}\n"
                elif new_exec["displayName"] is not None:
                    kc.create_subflow(
                        new_exec["displayName"], flow_alias_parent, realm=realm, flowType=new_exec["subFlowType"]
                    )
                    execution = kc.get_executions_representation(config, realm=realm)[exec_index]
                    exec_found = True
                    exec_index = new_exec_index
                    after += f"{new_exec}\n"
                if exec_found:
                    changed = True
                    if exec_index != -1:
                        # Update the existing execution
                        updated_exec = {"id": execution["id"]}
                        # add the execution configuration
                        if new_exec["authenticationConfig"] is not None:
                            if "authenticationConfig" in execution and "id" in execution["authenticationConfig"]:
                                kc.delete_authentication_config(execution["authenticationConfig"]["id"], realm=realm)
                            kc.add_authenticationConfig_to_execution(
                                updated_exec["id"], new_exec["authenticationConfig"], realm=realm
                            )
                        for key in new_exec:
                            # remove unwanted key for the next API call
                            if key not in ("flowAlias", "authenticationConfig", "subFlowType"):
                                updated_exec[key] = new_exec[key]
                        if new_exec["requirement"] is not None:
                            if "priority" in execution:
                                updated_exec["priority"] = execution["priority"]
                            kc.update_authentication_executions(flow_alias_parent, updated_exec, realm=realm)
                        diff = exec_index - new_exec_index
                        kc.change_execution_priority(updated_exec["id"], diff, realm=realm)
                        after += f"{kc.get_executions_representation(config, realm=realm)[new_exec_index]}\n"
        return changed, dict(before=before, after=after)
    except Exception as e:
        kc.module.fail_json(
            msg=f"Could not create or update executions for authentication flow {config['alias']} in realm {realm}: {e}"
        )


def main():
    """
    Module execution

    :return:
    """
    argument_spec = keycloak_argument_spec()

    meta_args = dict(
        realm=dict(type="str", required=True),
        alias=dict(type="str", required=True),
        providerId=dict(type="str", choices=["basic-flow", "client-flow"]),
        description=dict(type="str"),
        copyFrom=dict(type="str"),
        authenticationExecutions=dict(
            type="list",
            elements="dict",
            options=dict(
                providerId=dict(type="str"),
                displayName=dict(type="str"),
                requirement=dict(choices=["REQUIRED", "ALTERNATIVE", "DISABLED", "CONDITIONAL"], type="str"),
                flowAlias=dict(type="str"),
                authenticationConfig=dict(type="dict"),
                index=dict(type="int"),
                subFlowType=dict(choices=["basic-flow", "form-flow"], default="basic-flow", type="str"),
            ),
        ),
        state=dict(choices=["absent", "present"], default="present"),
        force=dict(type="bool", default=False),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    result = dict(changed=False, msg="", flow={})

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    realm = module.params.get("realm")
    state = module.params.get("state")
    force = module.params.get("force")

    new_auth_repr = {
        "alias": module.params.get("alias"),
        "copyFrom": module.params.get("copyFrom"),
        "providerId": module.params.get("providerId"),
        "authenticationExecutions": module.params.get("authenticationExecutions"),
        "description": module.params.get("description"),
        "builtIn": module.params.get("builtIn"),
        "subflow": module.params.get("subflow"),
    }

    auth_repr = kc.get_authentication_flow_by_alias(alias=new_auth_repr["alias"], realm=realm)

    # Cater for when it doesn't exist (an empty dict)
    if not auth_repr:
        if state == "absent":
            # Do nothing and exit
            if module._diff:
                result["diff"] = dict(before="", after="")
            result["changed"] = False
            result["end_state"] = {}
            result["msg"] = f"{new_auth_repr['alias']} absent"
            module.exit_json(**result)

        elif state == "present":
            # Process a creation
            result["changed"] = True

            if module._diff:
                result["diff"] = dict(before="", after=new_auth_repr)

            if module.check_mode:
                module.exit_json(**result)

            # If copyFrom is defined, create authentication flow from a copy
            if "copyFrom" in new_auth_repr and new_auth_repr["copyFrom"] is not None:
                auth_repr = kc.copy_auth_flow(config=new_auth_repr, realm=realm)
            else:  # Create an empty authentication flow
                auth_repr = kc.create_empty_auth_flow(config=new_auth_repr, realm=realm)

            # If the authentication still not exist on the server, raise an exception.
            if auth_repr is None:
                result["msg"] = f"Authentication just created not found: {new_auth_repr}"
                module.fail_json(**result)

            # Configure the executions for the flow
            create_or_update_executions(kc=kc, config=new_auth_repr, realm=realm)

            # Get executions created
            exec_repr = kc.get_executions_representation(config=new_auth_repr, realm=realm)
            if exec_repr is not None:
                auth_repr["authenticationExecutions"] = exec_repr
            result["end_state"] = auth_repr

    else:
        if state == "present":
            # Process an update

            if force:  # If force option is true
                # Delete the actual authentication flow
                result["changed"] = True
                if module._diff:
                    result["diff"] = dict(before=auth_repr, after=new_auth_repr)
                if module.check_mode:
                    module.exit_json(**result)
                kc.delete_authentication_flow_by_id(id=auth_repr["id"], realm=realm)
                # If copyFrom is defined, create authentication flow from a copy
                if "copyFrom" in new_auth_repr and new_auth_repr["copyFrom"] is not None:
                    auth_repr = kc.copy_auth_flow(config=new_auth_repr, realm=realm)
                else:  # Create an empty authentication flow
                    auth_repr = kc.create_empty_auth_flow(config=new_auth_repr, realm=realm)
                # If the authentication still not exist on the server, raise an exception.
                if auth_repr is None:
                    result["msg"] = f"Authentication just created not found: {new_auth_repr}"
                    module.fail_json(**result)
            # Configure the executions for the flow

            if module.check_mode:
                module.exit_json(**result)
            changed, diff = create_or_update_executions(kc=kc, config=new_auth_repr, realm=realm)
            result["changed"] |= changed

            if module._diff:
                result["diff"] = diff

            # Get executions created
            exec_repr = kc.get_executions_representation(config=new_auth_repr, realm=realm)
            if exec_repr is not None:
                auth_repr["authenticationExecutions"] = exec_repr
            result["end_state"] = auth_repr

        else:
            # Process a deletion (because state was not 'present')
            result["changed"] = True

            if module._diff:
                result["diff"] = dict(before=auth_repr, after="")

            if module.check_mode:
                module.exit_json(**result)

            # delete it
            kc.delete_authentication_flow_by_id(id=auth_repr["id"], realm=realm)

            result["msg"] = f"Authentication flow: {new_auth_repr['alias']} id: {auth_repr['id']} is deleted"

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The eos_ospfv2 class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_diff,
    remove_empties,
    to_list,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import Facts


class Ospfv2(ConfigBase):
    """
    The eos_ospfv2 class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["ospfv2"]

    def __init__(self, module):
        super(Ospfv2, self).__init__(module)

    def get_ospfv2_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )

        ospfv2_facts = facts["ansible_network_resources"].get("ospfv2")
        if not ospfv2_facts:
            return []
        return ospfv2_facts

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        warnings = list()
        commands = list()

        if self.state in self.ACTION_STATES:
            existing_ospfv2_facts = self.get_ospfv2_facts()
        else:
            existing_ospfv2_facts = []
        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_ospfv2_facts))
        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True
        if self.state in self.ACTION_STATES:
            result["commands"] = commands
        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_ospfv2_facts = self.get_ospfv2_facts()
        elif self.state == "rendered":
            result["rendered"] = commands
        elif self.state == "parsed":
            if not self._module.params["running_config"]:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_ospfv2_facts(
                data=self._module.params["running_config"],
            )
        else:
            changed_ospfv2_facts = self.get_ospfv2_facts()
        if self.state in self.ACTION_STATES:
            result["before"] = existing_ospfv2_facts
            if result["changed"]:
                result["after"] = changed_ospfv2_facts
        elif self.state == "gathered":
            result["gathered"] = changed_ospfv2_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_ospfv2_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_ospfv2_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        if self.state in ("merged", "replaced", "overridden", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    self.state,
                ),
            )
        if self.state == "overridden":
            commands = self._state_overridden(want, have)
        elif self.state == "deleted":
            commands = self._state_deleted(want, have)
        elif self.state == "merged" or self.state == "rendered":
            commands = self._state_merged(want, have)
        elif self.state == "replaced":
            commands = self._state_replaced(want, have)
        return commands

    def _get_os_version(self):
        os_version = "4.20"
        if self._connection():
            os_version = self._connection.get_device_info()["network_os_version"]
        return os_version

    def _state_replaced(self, want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for w in want["processes"]:
            del_cmds = w.copy()
            add_cmds = {}
            for h in have["processes"]:
                if h["process_id"] != w["process_id"]:
                    continue
                if w.get("vrf"):
                    if w["vrf"] != h["vrf"]:
                        self._module.fail_json(
                            msg="Value of vrf and process_id does not match the config present in the device",
                        )
                        break
                del_instance_list = self.compare_dicts(h, w)
                if del_instance_list:
                    del_cmds = {"processes": del_instance_list}
                add_instance_list = self.compare_dicts(w, h)
                if add_instance_list:
                    add_cmds = {"processes": add_instance_list}

            return_command = self.del_commands(del_cmds, have)
            for command in return_command:
                if "exit" not in command:
                    commands.append(command)
            return_command = self.add_commands(add_cmds, have)
            for command in return_command:
                if "router ospf" in command:
                    if command not in commands:
                        commands.append(command)
                else:
                    commands.append(command)
        commandset = []
        if commands:
            commandset.append(commands[0])
            for cmd in commands[1::]:
                if "router ospf" in cmd and commandset[-1] != "exit":
                    commandset.append("exit")
                commandset.append(cmd)
            if commandset[-1] != "exit":
                commandset.append("exit")
        return commandset

    def _state_overridden(self, want, have):
        """The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for h in have["processes"]:
            present = False
            for w in want["processes"]:
                if h["process_id"] == w["process_id"]:
                    present = True
                    break
            if not present:
                commands.append("no router ospf " + str(h["process_id"]))
        replace_cmds = self._state_replaced(want, have)
        for cmd in replace_cmds:
            commands.append(cmd)
        return commands

    def _state_merged(self, want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """

        return self.set_commands(want, have)

    def _state_deleted(self, want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        return_command = self.del_commands(want, have)
        if return_command:
            for cmd in return_command:
                if "no exit" in cmd:
                    cmd = "exit"
                commands.append(cmd)
        return commands

    def set_commands(self, want, have):
        commands = []
        instance_list = []
        for w in want["processes"]:
            present = False
            c = []
            if have and not have.get("processes"):
                instance_list = want["processes"]
                break
            if have:
                for h in have["processes"]:
                    if w["process_id"] == h["process_id"]:
                        if w.get("vrf"):
                            if w["vrf"] != h["vrf"]:
                                self._module.fail_json(
                                    msg="Value of vrf and process_id does not match the config present in the device",
                                )
                                continue
                        present = True
                        c = self.compare_dicts(w, h)
                        break
            if c:
                instance_list.append(c[0])
            if not present:
                if w["vrf"] in _get_vrf_list(have):
                    self._module.fail_json(
                        msg="Value of vrf and process_id does not match the config present in the device",
                    )
                instance_list.append(w)
        instance_dict = {"processes": instance_list}
        return_command = self.add_commands(instance_dict, have)
        for command in return_command:
            commands.append(command)
        return commands

    def compare_dicts(self, want_inst, have_inst):
        want_dict = remove_empties(want_inst)
        have = have_inst
        ospf_list = []
        return_ospf_dict = {}
        for w_key in want_dict.keys():
            if not have.get(w_key):
                return_ospf_dict.update({w_key: want_dict[w_key]})
            elif (
                isinstance(want_dict[w_key], str)
                or isinstance(want_dict[w_key], bool)
                or isinstance(want_dict[w_key], int)
            ):
                if want_dict[w_key] != have[w_key]:
                    return_ospf_dict.update({w_key: want_dict[w_key]})
            elif isinstance(want_dict[w_key], dict):
                diff = dict_diff(have.get(w_key, {}), want_dict[w_key])
                if diff:
                    return_ospf_dict.update({w_key: diff})
            elif isinstance(want_dict[w_key], list):
                if have.get(w_key):
                    compare_list = self.compare_ospf_list(
                        want_dict[w_key],
                        have.get(w_key),
                        w_key,
                    )
                    if compare_list:
                        return_ospf_dict.update({w_key: compare_list})
            else:
                if want_dict[w_key] != have.get(w_key):
                    return_ospf_dict.update({w_key: want_dict[w_key]})

        if return_ospf_dict:
            if want_dict.get("vrf"):
                return_ospf_dict.update(
                    {
                        "process_id": want_dict["process_id"],
                        "vrf": want_dict["vrf"],
                    },
                )
            else:
                return_ospf_dict.update(
                    {"process_id": want_dict["process_id"]},
                )
            ospf_list.append(return_ospf_dict)
        return ospf_list

    def compare_ospf_list(self, w_list, h_list, l_key):
        return_list = []
        for w in w_list:
            present = False
            for h in h_list:
                diff = dict_diff(h, w)
                if not diff:
                    present = True
                    break
            if not present:
                return_list.append(w)
        return return_list

    def add_commands(self, want, have):
        commands = []
        if not want:
            return commands
        for ospf_params in want["processes"]:
            commands.append(_get_router_command(ospf_params))
            if ospf_params.get("traffic_engineering"):
                commands.append("traffic-engineering")
            if ospf_params.get("adjacency"):
                threshold = ospf_params["adjacency"]["exchange_start"]["threshold"]
                commands.append(
                    "adjacency exchange-start threshold " + str(threshold),
                )
            if ospf_params.get("areas"):
                command_list = _parse_areas(ospf_params["areas"])
                for c in command_list:
                    commands.append(c)
            if ospf_params.get("auto_cost"):
                commands.append(
                    "auto-cost reference-bandwidth "
                    + str(ospf_params["auto_cost"]["reference_bandwidth"]),
                )
            if ospf_params.get("bfd"):
                os_version = self._get_os_version()
                if os_version < "4.23":
                    commands.append("bfd all-interfaces")
                else:
                    commands.append("bfd default")
            if ospf_params.get("default_information"):
                commands.append(
                    _parse_default_information(
                        ospf_params["default_information"],
                    ),
                )
            if ospf_params.get("default_metric"):
                commands.append(
                    "default-metric" + " " + str(ospf_params["default_metric"]),
                )
            if ospf_params.get("distance"):
                for k, v in ospf_params["distance"].items():
                    if v:
                        k = re.sub(r"_", "-", k)
                        commands.append("distance ospf " + k + " " + str(v))
            if ospf_params.get("distribute_list"):
                commands.append(
                    "distribute-list "
                    + ospf_params["distribute_list"].keys()[0]
                    + " "
                    + ospf_params["distribute_list"].values()[0]
                    + " in",
                )
            if ospf_params.get("dn_bit_ignore"):
                commands.append("dn-bit-ignore")
            if ospf_params.get("graceful_restart"):
                if ospf_params["graceful_restart"].get("set"):
                    commands.append("graceful-restart")
                else:
                    commands.append(
                        "graceful-restart grace-period "
                        + str(
                            ospf_params["graceful_restart"].get(
                                "grace_period",
                            ),
                        ),
                    )
            if ospf_params.get("graceful_restart_helper"):
                commands.append("graceful-restart-helper")
            if ospf_params.get("log_adjacency_changes"):
                cmd = "log-adjacency-changes"
                if ospf_params["log_adjacency_changes"].get("detail"):
                    cmd = cmd + " detail"
                commands.append(cmd)
            if ospf_params.get("max_lsa"):
                commands.append(_parse_max_lsa(ospf_params["max_lsa"]))
            if ospf_params.get("max_metric"):
                commands.append(_parse_max_metric(ospf_params["max_metric"]))
            if ospf_params.get("maximum_paths"):
                commands.append(
                    "maximum-paths " + str(ospf_params["maximum_paths"]),
                )
            if ospf_params.get("mpls_ldp"):
                commands.append("mpls ldp sync default")
            if ospf_params.get("networks"):
                command_list = _parse_networks(ospf_params["networks"])
                for c in command_list:
                    commands.append(c)
            if ospf_params.get("passive_interface"):
                if "interface_list" in ospf_params["passive_interface"].keys():
                    commands.append(
                        "passive-interface " + ospf_params["passive_interface"]["interface_list"],
                    )
                else:
                    commands.append("passive-interface default")
            if ospf_params.get("point_to_point"):
                commands.append("point-to-point routes")
            if ospf_params.get("redistribute"):
                command_list = _parse_redistribute(ospf_params["redistribute"])
                for c in command_list:
                    commands.append(c)
            if ospf_params.get("retransmission_threshold"):
                commands.append(
                    "retransmission-threshold lsa " + str(ospf_params["retransmission_threshold"]),
                )
            if ospf_params.get("rfc1583compatibility"):
                commands.append("compatible rfc1583")
            if ospf_params.get("router_id"):
                commands.append("router-id " + ospf_params.get("router_id"))
            if ospf_params.get("summary_address"):
                commands.append(
                    _parse_summary_address(ospf_params["summary_address"]),
                )
            if ospf_params.get("timers"):
                os_version = self._get_os_version()
                command_list = _parse_timers(ospf_params["timers"], os_version)
                for c in command_list:
                    commands.append(c)
            commands.append("exit")
        commandset = []
        for command in commands:
            commandset.append(command.strip())
        return commandset

    def del_commands(self, want, have):
        commands = []
        other_commands = 0
        want = remove_empties(want)
        if want.get("processes"):
            for w_inst in want["processes"]:
                router_context = 0
                d_cmds = []
                instance_list = []
                if have.get("processes"):
                    for h_inst in have["processes"]:
                        if h_inst["process_id"] == w_inst["process_id"]:
                            if w_inst.get("vrf") and w_inst.get(
                                "vrf",
                            ) == h_inst.get("vrf"):
                                if list(w_inst.keys()) == [
                                    "process_id",
                                    "vrf",
                                ]:
                                    commands.append(
                                        "no router ospf "
                                        + str(w_inst["process_id"])
                                        + " vrf "
                                        + w_inst["vrf"],
                                    )
                                    router_context = 1
                            if len(w_inst.keys()) == 1 and list(
                                w_inst.keys(),
                            ) == ["process_id"]:
                                commands.append(
                                    "no router ospf " + str(w_inst["process_id"]),
                                )
                                router_context = 1
                            if not router_context:
                                instance_list = self.compare_dicts(
                                    w_inst,
                                    h_inst,
                                )
                                if not instance_list:
                                    del_want = {"processes": [w_inst]}
                                    d_cmds = self.add_commands(del_want, have)
                                for cmd in d_cmds:
                                    if "router ospf" in cmd:
                                        other_commands = 0
                                        if cmd not in commands:
                                            commands.append(cmd)
                                    else:
                                        cmd = "no " + cmd
                                        if cmd not in commands:
                                            commands.append(cmd)
                                            other_commands += 1
                if not other_commands and len(commands) == 1 and not router_context:
                    if "no" not in commands[0] and "router ospf" in commands[0]:
                        commands[0] = "no " + commands[0]
        return commands


def _get_router_command(inst):
    command = ""
    if inst.get("vrf") and inst.get("vrf") != "default":
        command = "router ospf " + str(inst["process_id"]) + " vrf " + inst["vrf"]
    else:
        command = "router ospf " + str(inst["process_id"])
    return command


def _get_vrf_list(want):
    vrf_list = []
    if not want:
        return vrf_list
    for w in want["processes"]:
        if w.get("vrf"):
            vrf_list.append(w["vrf"])
    return vrf_list


def _parse_areas(areas):
    command = []
    for area in areas:
        area_cmd = "area " + area["area_id"]
        if area.get("default_cost"):
            command.append(
                area_cmd + " default-cost " + str(area.get("default_cost")),
            )
        elif area.get("filter"):
            command.append(
                area_cmd + " " + _parse_areas_filter(area["filter"]),
            )
        elif area.get("not_so_stubby"):
            command.append(
                area_cmd + " " + _parse_areas_filter_notsostubby(area["not_so_stubby"]),
            )
        elif area.get("nssa"):
            command.append(
                area_cmd + " " + _parse_areas_filter_nssa(area["nssa"]),
            )
        elif area.get("range"):
            command.append(area_cmd + " " + _parse_areas_range(area["range"]))
    return command


def _parse_areas_filter(filter_dict):
    filter_cmd = "filter "
    if filter_dict.get("prefix_list"):
        filter_cmd = filter_cmd + filter_dict.get("filter")
    elif filter_dict.get("address"):
        filter_cmd = filter_cmd + filter_dict.get("address")
    else:
        filter_cmd = (
            filter_cmd + filter_dict.get("subnet_address") + " " + filter_dict.get("subnet_mask")
        )
    return filter_cmd


def _parse_areas_filter_notsostubby(nss_dict):
    nss_cmd = "not-so-stubby "
    if nss_dict.get("default_information_originate"):
        nss_cmd = nss_cmd + "default-information-originate "
        for def_keys in nss_dict["default_information_originate"].keys():
            if def_keys == "nssa_only" and nss_dict["default_information_originate"]["nssa_only"]:
                nss_cmd = nss_cmd + " nssa-only "
            elif nss_dict["default_information_originate"].get(def_keys):
                nss_cmd = (
                    nss_cmd + def_keys + " " + nss_dict["default_information_originate"][def_keys]
                )
    elif "lsa" in nss_dict.keys() and nss_dict.get("lsa"):
        nss_cmd = nss_cmd + " lsa type-7 convert type-5"
    elif "no_summary" in nss_dict.keys() and nss_dict.get("no_summary"):
        nss_cmd = nss_cmd + " no-summary"
    elif "nssa_only" in nss_dict.keys() and nss_dict.get("nssa_only"):
        nss_cmd = nss_cmd + " nssa-only"
    return nss_cmd


def _parse_areas_filter_nssa(nss_dict):
    nss_cmd = "nssa "
    if nss_dict.get("default_information_originate"):
        nss_cmd = nss_cmd + "default-information-originate "
        for def_keys in nss_dict["default_information_originate"].keys():
            if def_keys == "nssa_only" and nss_dict["default_information_originate"]["nssa_only"]:
                nss_cmd = nss_cmd + " nssa-only "
            elif nss_dict["default_information_originate"].get(def_keys):
                nss_cmd = (
                    nss_cmd + def_keys + " " + nss_dict["default_information_originate"][def_keys]
                )
    elif "no_summary" in nss_dict.keys() and nss_dict.get("no_summary"):
        nss_cmd = nss_cmd + " no-summary"
    elif "nssa_only" in nss_dict.keys() and nss_dict.get("nssa_only"):
        nss_cmd = nss_cmd + " nssa-only"
    return nss_cmd


def _parse_areas_range(range_dict):
    range_cmd = " range "
    if range_dict.get("address"):
        range_cmd = range_cmd + range_dict["address"]
    if range_dict.get("subnet_address"):
        range_cmd = range_cmd + range_dict["subnet_address"] + " " + range_dict["subnet_mask"]
    if range_dict.get("advertise") is not None:
        if range_dict["advertise"]:
            range_cmd = range_cmd + " advertise "
        else:
            range_cmd = range_cmd + " not-advertise "
    if range_dict.get("cost"):
        range_cmd = range_cmd + " cost " + str(range_dict["cost"])
    return range_cmd


def _parse_default_information(default_dict):
    def_cmd = "default-information originate"
    for def_key in sorted(default_dict.keys()):
        if def_key == "always":
            if default_dict.get(def_key):
                def_cmd = def_cmd + " " + def_key
        elif def_key in ["metric", "metric_type", "route_map"]:
            if default_dict.get(def_key):
                k = re.sub(r"_", "-", def_key)
                def_cmd = def_cmd + " " + k + " " + str(default_dict[def_key])
    return def_cmd


def _parse_max_lsa(max_lsa_dict):
    max_lsa_cmd = "max-lsa "
    if max_lsa_dict.get("count"):
        max_lsa_cmd = max_lsa_cmd + " " + str(max_lsa_dict["count"])
    if max_lsa_dict.get("threshold"):
        max_lsa_cmd = max_lsa_cmd + " " + str(max_lsa_dict["threshold"])
    for lsa_key, lsa_val in sorted(max_lsa_dict.items()):
        if lsa_key == "warning" and lsa_val:
            max_lsa_cmd = max_lsa_cmd + " warning-only"
        elif lsa_key in ["ignore_count", "reset_time", "ignore_time"]:
            if lsa_val:
                k = re.sub(r"_", "-", lsa_key)
                max_lsa_cmd = max_lsa_cmd + " " + k + " " + str(lsa_val) + " "
    return max_lsa_cmd


def _parse_max_metric(max_metric_dict):
    metric_cmd = "max-metric router-lsa "
    for k, v in max_metric_dict["router_lsa"].items():
        if not v:
            continue
        if k == "include_stub" and v:
            metric_cmd = metric_cmd + " include-stub"
        elif k == "on_startup":
            metric_cmd = metric_cmd + " on-startup " + str(v["wait_period"])
        elif k in ["summary_lsa", "external_lsa"]:
            k = re.sub(r"_", "-", k)
            if v.get("set"):
                metric_cmd = metric_cmd + " " + k
            else:
                metric_cmd = metric_cmd + " " + k + " " + str(v.get("max_metric_value"))
    return metric_cmd


def _parse_networks(net_list):
    network_cmd = []
    for net_dict in net_list:
        net_cmd = "network "
        if net_dict.get("prefix"):
            net_cmd = net_cmd + net_dict.get("prefix")
        else:
            net_cmd = net_cmd + net_dict.get("network_address") + " " + net_dict.get("mask")
        if net_dict.get("area"):
            net_cmd = net_cmd + " area " + net_dict.get("area")
        network_cmd.append(net_cmd)
    return network_cmd


def _parse_redistribute(r_list):
    rcmd_list = []
    for r_dict in r_list:
        r_cmd = "redistribute "
        r_cmd = r_cmd + r_dict["routes"]
        if r_dict.get("isis_level"):
            k = re.sub(r"_", "-", r_dict["isis_level"])
            r_cmd = r_cmd + " " + k
        if r_dict.get("route_map"):
            r_cmd = r_cmd + " route-map " + r_dict["route_map"]
        rcmd_list.append(r_cmd)
    return rcmd_list


def _parse_summary_address(addr_dict):
    sum_cmd = "summary-address "
    if addr_dict.get("prefix"):
        sum_cmd = sum_cmd + addr_dict.get("prefix")
    else:
        sum_cmd = sum_cmd + addr_dict.get("address") + " " + addr_dict.get("mask")
    if "attribute_map" in addr_dict.keys():
        sum_cmd = sum_cmd + " attribute-map " + addr_dict["attribute_map"]
    elif addr_dict.get("not_advertise"):
        sum_cmd = sum_cmd + " not-advertise "
    elif "tag" in addr_dict.keys():
        sum_cmd = sum_cmd + " tag " + addr_dict["tag"]
    return sum_cmd


def _parse_timers(timers_list, os_version="4.20"):
    timers_cmd = []
    for t_dict in timers_list:
        t_cmd = "timers "
        for t_key in t_dict.keys():
            if not t_dict.get(t_key):
                break
            if t_key == "lsa":
                if t_dict["lsa"].get("rx"):
                    if os_version < "4.23":
                        t_cmd = t_cmd + "lsa arrival " + str(t_dict["lsa"]["rx"]["min_interval"])
                    else:
                        t_cmd = (
                            t_cmd
                            + "lsa rx min interval "
                            + str(t_dict["lsa"]["rx"]["min_interval"])
                        )
                else:
                    t_cmd = (
                        t_cmd
                        + "lsa tx delay initial "
                        + str(t_dict["lsa"]["tx"]["delay"]["initial"])
                        + " "
                        + str(t_dict["lsa"]["tx"]["delay"]["min"])
                        + " "
                        + str(t_dict["lsa"]["tx"]["delay"]["max"])
                    )
            elif t_key == "out_delay":
                t_cmd = t_cmd + " out-delay " + str(t_dict["out_delay"])
            elif t_key == "pacing":
                t_cmd = t_cmd + " pacing flood " + str(t_dict["pacing"])
            elif t_key == "spf":
                if "seconds" in t_dict["spf"].keys():
                    t_cmd = t_cmd + " spf " + str(t_dict["spf"]["seconds"])
                else:
                    t_cmd = (
                        t_cmd
                        + "spf delay initial "
                        + str(t_dict["spf"]["initial"])
                        + " "
                        + str(t_dict["spf"]["max"])
                        + " "
                        + str(t_dict["spf"]["min"])
                    )
            elif t_key == "throttle":
                if t_dict["throttle"]["attr"] == "lsa":
                    t_cmd = t_cmd + "throttle lsa all "
                else:
                    t_cmd = t_cmd + "throttle spf "
                t_cmd = (
                    t_cmd
                    + str(t_dict["throttle"]["initial"])
                    + " "
                    + str(t_dict["throttle"]["min"])
                    + " "
                    + str(t_dict["throttle"]["max"])
                )
            timers_cmd.append(t_cmd)
    return timers_cmd



#!/usr/bin/python
#
# Copyright (c) 2025 Klaas Demter (@Klaas-)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_monitoractiongroups
version_added: "3.7.0"
short_description: Create, update and delete Action Groups.
description:
    - Create, update and delete Action Groups.
    - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.actiongroupresource?view=azure-python)

options:
    name:
        description:
            - The name of the action group you're creating/changing.
        required: true
        type: str
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    location:
        description:
            - Location of the action group.
            - defaults to location of exiting action group or location of the resource group if unspecified.
            - Action groups are usually 'Global'.
        required: false
        type: str
    group_short_name:
        description:
            - The short name of the action group. This will be used in SMS messages.
        type: str
    enabled:
        description:
            - Indicates whether this action group is enabled. If an action group is not enabled, then none of its receivers will receive communications.
        type: bool
    email_receivers:
        description:
            - The list of email receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.emailreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            email_address:
                description:
                    - The email address of this receiver.
                    - Required when setting this receiver.
                type: str
            use_common_alert_schema:
                description:
                    - Indicates whether to use common alert schema.
                    - Defaults to False when not set (server side default).
                type: bool
    sms_receivers:
        description:
            - The list of SMS receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.smsreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            country_code:
                description:
                    - The country code of the SMS receiver.
                    - Required when setting this receiver.
                type: str
            phone_number:
                description:
                    - The phone number of the SMS receiver.
                    - Required when setting this receiver.
                type: str
    webhook_receivers:
        description:
            - The list of webhook receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.webhookreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            service_uri:
                description:
                    - The URI where webhooks should be sent.
                    - Required when setting this receiver.
                type: str
            use_common_alert_schema:
                description:
                    - Indicates whether to use common alert schema.
                    - Defaults to False when not set (server side default).
                type: bool
            use_aad_auth:
                description:
                    - Indicates whether or not use AAD authentication.
                    - Defaults to False when not set (server side default).
                type: bool
            object_id:
                description:
                    - Indicates the webhook app object Id for aad auth.
                type: str
            identifier_uri:
                description:
                    - Indicates the identifier uri for aad auth.
                type: str
            tenant_id:
                description:
                    - Indicates the tenant id for aad auth.
                type: str
    itsm_receivers:
        description:
            - The list of webhook receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.itsmreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            workspace_id:
                description:
                    - OMS LA instance identifier.
                    - Required when setting this receiver.
                type: str
            connection_id:
                description:
                    - Unique identification of ITSM connection among multiple defined in above workspace.
                    - Required when setting this receiver.
                type: str
            ticket_configuration:
                description:
                    - JSON blob for the configurations of the ITSM action. CreateMultipleWorkItems option will be part of this blob as well.
                    - Required when setting this receiver.
                type: str
            region:
                description:
                    - Region in which workspace resides.
                    - Required when setting this receiver.
                type: str
    azure_app_push_receivers:
        description:
            - The list of AzureAppPush receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.azureapppushreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name of this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            email_address:
                description:
                    - The email address registered for the Azure mobile app.
                    - Required when setting this receiver.
                type: str
    automation_runbook_receivers:
        description:
            - The list of AutomationRunbook receivers that are part of this action group.
            - >-
              U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.automationrunbookreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            automation_account_id:
                description:
                    - The Azure automation account Id which holds this runbook and authenticate to Azure resource.
                    - Required when setting this receiver.
                type: str
            runbook_name:
                description:
                    - The name for this runbook.
                    - Required when setting this receiver.
                type: str
            webhook_resource_id:
                description:
                    - The resource id for webhook linked to this runbook.
                    - Required when setting this receiver.
                type: str
            is_global_runbook:
                description:
                    - Indicates whether this instance is global runbook.
                type: bool
            name:
                description:
                    - Indicates name of the webhook.
                type: str
            service_uri:
                description:
                    - The URI where webhooks should be sent.
                type: str
            use_common_alert_schema:
                description:
                    - Indicates whether to use common alert schema.
                    - Defaults to False when not set (server side default).
                type: bool
    voice_receivers:
        description:
            - The list of voice receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.voicereceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            country_code:
                description:
                    - The country code of the voice receiver.
                    - Required when setting this receiver.
                type: str
            phone_number:
                description:
                    - The phone number of the voice receiver.
                    - Required when setting this receiver.
                type: str
    logic_app_receivers:
        description:
            - The list of logic app receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.logicappreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            resource_id:
                description:
                    - The azure resource id of the logic app receiver.
                    - Required when setting this receiver.
                type: str
            callback_url:
                description:
                    - The callback url where http request sent to.
                    - Required when setting this receiver.
                type: str
            use_common_alert_schema:
                description:
                    - Indicates whether to use common alert schema.
                    - Defaults to False when not set (server side default).
                type: bool
    azure_function_receivers:
        description:
            - The list of azure function receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.azurefunctionreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            function_app_resource_id:
                description:
                    - The azure resource id of the function app.
                    - Required when setting this receiver.
                type: str
            function_name:
                description:
                    - The function name in the function app.
                    - Required when setting this receiver.
                type: str
            http_trigger_url:
                description:
                    - The http trigger url where http request sent to.
                    - Required when setting this receiver.
                type: str
            use_common_alert_schema:
                description:
                    - Indicates whether to use common alert schema.
                    - Defaults to False when not set (server side default).
                type: bool
    arm_role_receivers:
        description:
            - The list of ARM role receivers that are part of this action group. Roles are Azure RBAC roles and only built-in roles are supported.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.armrolereceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            role_id:
                description:
                    - The arm role id.
                    - Required when setting this receiver.
                type: str
            use_common_alert_schema:
                description:
                    - Indicates whether to use common alert schema.
                    - Defaults to False when not set (server side default).
                type: bool
    event_hub_receivers:
        description:
            - The list of event hub receivers that are part of this action group.
            - U(https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.eventhubreceiver?view=azure-python)
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - The name this receiver. Names must be unique across all receivers within an action group.
                    - Required when setting this receiver.
                type: str
            event_hub_name_space:
                description:
                    - The Event Hub namespace.
                    - Required when setting this receiver.
                type: str
            event_hub_name:
                description:
                    - The name of the specific Event Hub queue.
                    - Required when setting this receiver.
                type: str
            use_common_alert_schema:
                description:
                    - Indicates whether to use common alert schema.
                    - Defaults to False when not set (server side default).
                type: bool
            tenant_id:
                description:
                    - The tenant Id for the subscription containing this event hub.
                type: str
            subscription_id:
                description:
                    - The Id for the subscription containing this event hub.
                    - Required when setting this receiver.
                type: str
    state:
        description:
            - State of the action group.
            - Use C(present) for creating/updating a action group.
            - Use C(absent) for deleting a action group.
        default: present
        type: str
        choices:
            - present
            - absent
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Klaas Demter (@Klaas-)
'''

EXAMPLES = '''
- name: Add a action group
  azure.azcollection.azure_rm_monitoractiongroups:
    state: present
    name: action_group_name
    resource_group: resource_group_name
    enabled: true
    location: Global
    email_receivers:
      - email_address: xxx@hostname.tld
        name: sendmail
        use_common_alert_schema: true
    group_short_name: short_name
    append_tags: false
    tags:
      ThisIsAnExampleTag: ExampleValue

- name: Add tag to existing action group
  azure.azcollection.azure_rm_monitoractiongroups:
    state: present
    name: action_group_name
    resource_group: resource_group_name
    append_tags: true
    tags:
      ThisIsAnAddedExampleTag: ExampleValue

- name: Remove all tags on existing action group
  azure.azcollection.azure_rm_monitoractiongroups:
    state: present
    name: action_group_name
    resource_group: resource_group_name
    append_tags: false

- name: Delete a action group
  azure.azcollection.azure_rm_monitoractiongroups:
    state: absent
    name: action_group_name
    resource_group: resource_group_name
'''

RETURN = '''
actiongroup:
    description:
        - Details of the action group
        - Is null on state==absent (action group does not exist or will be deleted)
        - Assumes you make legal changes in check mode
    type: dict
    returned: always
    sample: {
        "arm_role_receivers": [],
        "automation_runbook_receivers": [],
        "azure_app_push_receivers": [],
        "azure_function_receivers": [],
        "email_receivers": [
            {
                "email_address": "xxx@hostname.tld",
                "name": "sendmail",
                "use_common_alert_schema": true
            }
        ],
        "enabled": true,
        "event_hub_receivers": [],
        "group_short_name": "shortname",
        "id": \
"/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/resougce_group_name/providers/microsoft.insights/actionGroups/action_group_name",
        "itsm_receivers": [],
        "location": "Global",
        "logic_app_receivers": [],
        "name": "action_group_name",
        "sms_receivers": [],
        "tags": {},
        "type": "Microsoft.Insights/ActionGroups",
        "voice_receivers": [],
        "webhook_receivers": []
    }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.exceptions import ResourceNotFoundError

except ImportError:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'ActionGroup'

email_receivers_spec = dict(
    name=dict(type='str'),
    email_address=dict(type='str'),
    use_common_alert_schema=dict(type='bool')
)

sms_receivers_spec = dict(
    name=dict(type='str'),
    country_code=dict(type='str'),
    phone_number=dict(type='str')
)

webhook_receivers_spec = dict(
    name=dict(type='str'),
    service_uri=dict(type='str'),
    use_common_alert_schema=dict(type='bool'),
    use_aad_auth=dict(type='bool'),
    object_id=dict(type='str'),
    identifier_uri=dict(type='str'),
    tenant_id=dict(type='str')
)

itsm_receivers_spec = dict(
    name=dict(type='str'),
    workspace_id=dict(type='str'),
    connection_id=dict(type='str'),
    ticket_configuration=dict(type='str'),
    region=dict(type='str')
)

azure_app_push_receivers_spec = dict(
    name=dict(type='str'),
    email_address=dict(type='str')
)

automation_runbook_receivers_spec = dict(
    automation_account_id=dict(type='str'),
    runbook_name=dict(type='str'),
    webhook_resource_id=dict(type='str'),
    is_global_runbook=dict(type='bool'),
    name=dict(type='str'),
    service_uri=dict(type='str'),
    use_common_alert_schema=dict(type='bool')
)

voice_receivers_spec = dict(
    name=dict(type='str'),
    country_code=dict(type='str'),
    phone_number=dict(type='str')
)

logic_app_receivers_spec = dict(
    name=dict(type='str'),
    resource_id=dict(type='str'),
    callback_url=dict(type='str'),
    use_common_alert_schema=dict(type='bool')
)

azure_function_receivers_spec = dict(
    name=dict(type='str'),
    function_app_resource_id=dict(type='str'),
    function_name=dict(type='str'),
    http_trigger_url=dict(type='str'),
    use_common_alert_schema=dict(type='bool')
)

arm_role_receivers_spec = dict(
    name=dict(type='str'),
    role_id=dict(type='str'),
    use_common_alert_schema=dict(type='bool')
)

event_hub_receivers_spec = dict(
    name=dict(type='str'),
    event_hub_name_space=dict(type='str'),
    event_hub_name=dict(type='str'),
    use_common_alert_schema=dict(type='bool'),
    tenant_id=dict(type='str'),
    subscription_id=dict(type='str')
)


class AzureRMActionGroup(AzureRMModuleBaseExt):
    """Information class for an Azure RM Action Groups"""

    def __init__(self):
        # https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.actiongroupresource?view=azure-python
        self.module_arg_spec = dict(
            name=dict(type='str', required=True),
            resource_group=dict(type='str', required=True),
            location=dict(type='str'),
            group_short_name=dict(type='str'),
            enabled=dict(type='bool'),
            email_receivers=dict(type='list', elements='dict', options=email_receivers_spec),
            sms_receivers=dict(type='list', elements='dict', options=sms_receivers_spec),
            webhook_receivers=dict(type='list', elements='dict', options=webhook_receivers_spec),
            itsm_receivers=dict(type='list', elements='dict', options=itsm_receivers_spec),
            azure_app_push_receivers=dict(type='list', elements='dict', options=azure_app_push_receivers_spec),
            automation_runbook_receivers=dict(type='list', elements='dict', options=automation_runbook_receivers_spec),
            voice_receivers=dict(type='list', elements='dict', options=voice_receivers_spec),
            logic_app_receivers=dict(type='list', elements='dict', options=logic_app_receivers_spec),
            azure_function_receivers=dict(type='list', elements='dict', options=azure_function_receivers_spec),
            arm_role_receivers=dict(type='list', elements='dict', options=arm_role_receivers_spec),
            event_hub_receivers=dict(type='list', elements='dict', options=event_hub_receivers_spec),
            state=dict(type='str', choices=['present', 'absent'], default='present')
        )

        self.name = None
        self.resource_group = None
        self.location = None
        self.group_short_name = None
        self.enabled = None
        self.email_receivers = None
        self.sms_receivers = None
        self.webhook_receivers = None
        self.itsm_receivers = None
        self.azure_app_push_receivers = None
        self.automation_runbook_receivers = None
        self.voice_receivers = None
        self.logic_app_receivers = None
        self.azure_function_receivers = None
        self.arm_role_receivers = None
        self.event_hub_receivers = None
        self.state = None
        self.tags = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            actiongroup=dict(),
            diff=dict(
                before=None,
                after=None
            )
        )

        super(AzureRMActionGroup, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        # Defaults for variables
        result = None
        result_compare = dict(compare=[])
        before_dict = None

        # Get current action group if it exists
        before_dict = self.get_action_group()

        # Create dict from input, without None values
        # https://learn.microsoft.com/en-us/python/api/azure-mgmt-monitor/azure.mgmt.monitor.v2023_01_01.models.actiongroupresource?view=azure-python
        action_group_template = {
            "location": self.location,
            "group_short_name": self.group_short_name,
            "enabled": self.enabled,
            "email_receivers": self.email_receivers,
            "sms_receivers": self.sms_receivers,
            "webhook_receivers": self.webhook_receivers,
            "itsm_receivers": self.itsm_receivers,
            "azure_app_push_receivers": self.azure_app_push_receivers,
            "automation_runbook_receivers": self.automation_runbook_receivers,
            "voice_receivers": self.voice_receivers,
            "logic_app_receivers": self.logic_app_receivers,
            "azure_function_receivers": self.azure_function_receivers,
            "arm_role_receivers": self.arm_role_receivers,
            "event_hub_receivers": self.event_hub_receivers,
        }
        # Filter out all None values
        action_group_input = {key: value for key, value in action_group_template.items() if value is not None}

        # Create/Update if state==present
        if self.state == 'present':
            if before_dict is None:
                # action group does not exist, create
                # On creation default to location of resource group unless otherwise noted in input variables
                if not self.location:
                    resource_group = self.get_resource_group(self.resource_group)
                    action_group_input['location'] = resource_group.location
                # On creation input == what we send to api
                action_group_update = action_group_input
                # Needs to be extended by tags if set
                if self.tags:
                    action_group_update['tags'] = self.tags
                self.results['changed'] = True
                if self.check_mode:
                    # Check mode, skipping actual creation
                    pass
                else:
                    create_response = self.create_or_update(action_group_update)
            else:
                # action group already exists, updating it
                # Dict for update is the union of existing object overwritten by input data
                action_group_update = before_dict | action_group_input

                # Enhanced with tags (special behaviour because of append_tags possibility)
                update_tags, update_tags_content = self.update_tags(before_dict.get('tags'))
                if update_tags or not self.default_compare({}, action_group_update, before_dict, '', result_compare):
                    action_group_update['tags'] = update_tags_content
                    self.results['changed'] = True
                    if self.check_mode:
                        # Check mode, skipping actual creation
                        pass
                    else:
                        create_response = self.create_or_update(action_group_update)

            if self.check_mode or not self.results['changed']:
                # When object was not updated or when running in check mode
                # assume action_group_update is resulting object
                result = action_group_update
            else:
                # otherwise take resulting new object from response of create call
                result = create_response

        # Delete action group if state is absent and it exists
        # if it doesn't exist, it's already absent
        elif self.state == 'absent' and before_dict is not None:
            self.results['changed'] = True
            if self.check_mode:
                # do not delete in check mode
                pass
            else:
                self.delete()

        self.results['diff']['before'] = before_dict
        self.results['diff']['after'] = result
        self.results['actiongroup'] = result

        return self.results

    def get_action_group(self):
        '''
        Gets the properties of the specified action group.

        :return: List of Action Groups
        '''
        self.log("Checking if action group {0} in resource group {1} is present".format(self.name,
                                                                                        self.resource_group))

        result = None
        response = None

        try:
            response = self.monitor_management_client_action_groups.action_groups.get(action_group_name=self.name,
                                                                                      resource_group_name=self.resource_group)
        except ResourceNotFoundError as ex:
            self.log("Could not find action group {0} in resource group {1}".format(self.name, self.resource_group))
        if response:
            result = self.serialize_obj(response, AZURE_OBJECT_CLASS)

        return result

    def create_or_update(self, action_group_update):
        result = None
        response = None
        try:
            response = self.monitor_management_client_action_groups.action_groups.create_or_update(resource_group_name=self.resource_group,
                                                                                                   action_group_name=self.name,
                                                                                                   action_group=action_group_update,
                                                                                                   logging_enable=False)
        except Exception as ex:
            self.fail("Error creating or update action group {0} in resource group {1}: {2}".format(self.name, self.resource_group, str(ex)))

        if response:
            result = self.serialize_obj(response, AZURE_OBJECT_CLASS)

        return result

    def delete(self):
        response = None
        try:
            response = self.monitor_management_client_action_groups.action_groups.delete(resource_group_name=self.resource_group,
                                                                                         action_group_name=self.name)
        except Exception as ex:
            self.fail("Error deleting action group {0} in resource group {1}: {2}".format(self.name, self.resource_group, str(ex)))

        return response


def main():
    """Main execution"""
    AzureRMActionGroup()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# Copyright (c) 2022, John Cant <a.johncant@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    keycloak_argument_spec,
)


def keycloak_clientsecret_module() -> AnsibleModule:
    """
    Returns an AnsibleModule definition for modules that interact with a client
    secret.

    :return: argument_spec dict
    """
    argument_spec = keycloak_argument_spec()

    meta_args = dict(
        realm=dict(default="master"),
        id=dict(type="str"),
        client_id=dict(type="str", aliases=["clientId"]),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [
                ["id", "client_id"],
                ["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"],
            ]
        ),
        required_together=([["auth_username", "auth_password"]]),
        mutually_exclusive=[["token", "auth_realm"], ["token", "auth_username"], ["token", "auth_password"]],
    )

    return module


def keycloak_clientsecret_module_resolve_params(module: AnsibleModule, kc: KeycloakAPI) -> tuple[str, dict[str, t.Any]]:
    """
    Given an AnsibleModule definition for keycloak_clientsecret_*, and a
    KeycloakAPI client, resolve the params needed to interact with the Keycloak
    client secret, looking up the client by clientId if necessary via an API
    call.

    :return: tuple of id, realm
    """

    realm = module.params.get("realm")
    id = module.params.get("id")
    client_id = module.params.get("client_id")

    # only lookup the client_id if id isn't provided.
    # in the case that both are provided, prefer the ID, since it is one
    # less lookup.
    if id is None:
        # Due to the required_one_of spec, client_id is guaranteed to not be None
        client = kc.get_client_by_clientid(client_id, realm=realm)

        if client is None:
            module.fail_json(msg=f"Client does not exist {client_id}")

        id = client["id"]

    return id, realm



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_resource_group
short_description: Resource Group configuration for Cisco Intersight
description:
  - Manages Resource Group configuration on Cisco Intersight.
  - Resource Groups define a set of resources that can be used for access control and organization assignment.
  - Users specify Intersight-managed UCS domains by their target name (as shown in the Intersight Targets page).
  - Optionally, specific servers (sub-targets) within a domain can be selected by serial number.
  - Resource Groups are account-level resources and are not scoped to an organization.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/resource/Group/model/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  name:
    description:
      - The name assigned to the Resource Group.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Resource Group.
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  resources:
    description:
      - List of Intersight-managed UCS domain resources to include in the Resource Group.
      - Required when O(state) is C(present).
      - Each entry specifies a domain by its target name as shown in the Intersight GUI Targets page.
      - When O(resources[].sub_targets) is provided, only the specified servers (by serial number) are included instead of the full domain.
      - Multiple entries can be mixed, some with sub-targets and some without.
    type: list
    elements: dict
    suboptions:
      domain_name:
        description:
          - The name of the Intersight-managed UCS domain target.
          - Must match the target name exactly as displayed in the Intersight Targets page.
        type: str
        required: true
      sub_targets:
        description:
          - List of server serial numbers to include as sub-targets within the domain.
          - When specified, only these servers are included instead of the entire domain.
          - Servers are automatically classified as Blades or RackUnits via the Intersight API.
        type: list
        elements: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create Resource Group for full UCS domains
  cisco.intersight.intersight_resource_group:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ucs-domains-group"
    description: "Resource group for two UCS domains"
    resources:
      - domain_name: "ucs-domain-1"
      - domain_name: "ucs-domain-2"
    state: present

- name: Create Resource Group with sub-targets (specific servers)
  cisco.intersight.intersight_resource_group:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "specific-servers-group"
    description: "Only specific servers from the domain"
    resources:
      - domain_name: "ucs-domain-1"
        sub_targets:
          - "FCH26387BD1"
          - "FCH243974ZD"
          - "WZP26030TAV"
    state: present

- name: Create Resource Group with mixed resources
  cisco.intersight.intersight_resource_group:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "mixed-resources-group"
    description: "Full domain and specific servers from another"
    resources:
      - domain_name: "ucs-domain-1"
      - domain_name: "ucs-domain-2"
        sub_targets:
          - "FCH26387BD1"
          - "WZP26030TAV"
    state: present

- name: Delete Resource Group
  cisco.intersight.intersight_resource_group:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "ucs-domains-group"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "ucs-domains-group",
        "ObjectType": "resource.Group",
        "Description": "Resource group for two UCS domains",
        "Moid": "507f1f77bcf86cd799439099"
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def resolve_domain_device_registration(intersight, domain_name):
    """Resolve a domain target name to its DeviceRegistration Moid."""
    intersight.get_resource(
        resource_path='/asset/Targets',
        query_params={
            '$filter': f"Name eq '{domain_name}' and TargetType eq 'UCSFIISM'",
            '$select': 'Name,RegisteredDevice',
            '$expand': 'RegisteredDevice($select=Moid)',
        },
    )

    target = intersight.result['api_response']
    if not target.get('Moid'):
        intersight.module.fail_json(
            msg=f"Intersight-managed UCS domain target '{domain_name}' not found. "
                "Verify the target name matches exactly as shown in the Intersight Targets page."
        )

    registered_device = target.get('RegisteredDevice', {})
    device_reg_moid = registered_device.get('Moid')
    if not device_reg_moid:
        intersight.module.fail_json(
            msg=f"Domain target '{domain_name}' has no associated DeviceRegistration. "
                "Ensure the domain is properly claimed and connected."
        )

    return device_reg_moid


def classify_serial_numbers(intersight, serials):
    """Classify serial numbers as Blade or RackUnit via PhysicalSummaries.

    Returns:
        Tuple of (blade_serials, rack_serials) lists.
    """
    serials_filter = ",".join(f"'{s}'" for s in serials)
    intersight.get_resource(
        resource_path='/compute/PhysicalSummaries',
        query_params={
            '$filter': f"Serial in ({serials_filter}) and ManagementMode eq 'Intersight'",
            '$select': 'Serial,SourceObjectType',
        },
        return_list=True,
    )

    results = intersight.result['api_response']
    if not isinstance(results, list):
        results = [results] if results else []

    found_serials = {}
    for entry in results:
        serial = entry.get('Serial')
        obj_type = entry.get('SourceObjectType', '')
        if serial:
            found_serials[serial] = obj_type

    missing = set(serials) - set(found_serials.keys())
    if missing:
        intersight.module.fail_json(
            msg=f"The following serial numbers were not found as Intersight-managed servers: {sorted(missing)}"
        )

    blade_serials = [s for s in serials if 'Blade' in found_serials[s]]
    rack_serials = [s for s in serials if 'RackUnit' in found_serials[s]]

    return blade_serials, rack_serials


def resolve_resources_to_selectors(intersight, resources):
    """Resolve user-provided resources into API selector strings."""
    device_reg_moids = []
    all_blade_serials = []
    all_rack_serials = []

    for resource in resources:
        domain_name = resource['domain_name']
        sub_targets = resource.get('sub_targets')

        if sub_targets:
            blade_serials, rack_serials = classify_serial_numbers(intersight, sub_targets)
            all_blade_serials.extend(blade_serials)
            all_rack_serials.extend(rack_serials)
        else:
            moid = resolve_domain_device_registration(intersight, domain_name)
            device_reg_moids.append(moid)

    selectors = []

    if device_reg_moids:
        moids_str = ",".join(f"'{m}'" for m in device_reg_moids)
        selectors.append(
            f"/api/v1/asset/DeviceRegistrations?$filter=Moid in ({moids_str})"
        )

    if all_blade_serials:
        serials_str = ",".join(f"'{s}'" for s in all_blade_serials)
        selectors.append(
            f"/api/v1/compute/Blades?$filter=Serial in ({serials_str}) and ManagementMode eq 'Intersight'"
        )

    if all_rack_serials:
        serials_str = ",".join(f"'{s}'" for s in all_rack_serials)
        selectors.append(
            f"/api/v1/compute/RackUnits?$filter=Serial in ({serials_str}) and ManagementMode eq 'Intersight'"
        )

    return selectors


def build_api_body(intersight):
    """Build the API body for Resource Group configuration."""
    params = intersight.module.params
    if params['state'] == 'present':
        selectors = resolve_resources_to_selectors(intersight, params['resources'])

        intersight.api_body = {
            'Name': params['name'],
            'Qualifier': 'Allow-Selectors',
            'Selectors': [{'Selector': s} for s in selectors],
        }

        intersight.set_tags_and_description()


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        resources=dict(type='list', elements='dict', options=dict(
            domain_name=dict(type='str', required=True),
            sub_targets=dict(type='list', elements='str'),
        )),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'present', ['resources']],
        ],
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    build_api_body(intersight)

    resource_path = '/resource/Groups'

    intersight.configure_secondary_resource(
        resource_path=resource_path,
        resource_name=module.params['name'],
        state=module.params['state'],
    )

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2016 Matt Davis, <mdavis@ansible.com>
# Copyright: (c) 2016 Chris Houseknecht, <house@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


class ModuleDocFragment(object):

    # Azure doc fragment
    DOCUMENTATION = r'''
options:
    plugin:
        description: marks this as an instance of the 'azure_rm' plugin
        required: true
        choices: ['azure_kql', 'azure.azcollection.azure_kql']
    graph_query:
        description: A graph query which will retrieve the inventory of hosts you are interested in.
            You must return inventory_hostname as a field from your query.
    fail_on_template_errors:
        description: When false, template failures during group and filter processing are silently ignored (eg,
            if a filter or group expression refers to an undefined host variable)
        choices: [True, False]
        default: True
    keyed_groups:
        description: Creates groups based on the value of a host variable. Requires a list of dictionaries,
            defining C(key) (the source dictionary-typed variable), C(prefix) (the prefix to use for the new group
            name), and optionally C(separator) (which defaults to C(_))
    groups:
        description: A mapping of group names to Jinja2 expressions. When the mapped expression is true, the host
            is added to the named group.
    compose:
        description: A mapping of hostvar names to Jinja2 expressions. The value for each host is the result of the
            Jinja2 expression (which may refer to any of the host's existing variables at the time this inventory
            plugin runs).
    hostnames:
        description:
        - A list of Jinja2 expressions in order of precedence to compose inventory_hostname.
        - Ignores expression if result is an empty string or None value.
        - An expression of C(default) will force using the default hostname generator if no previous hostname expression
            resulted in a valid hostname.
        - Use C(default_inventory_hostname) to access the default hostname generator's value in any of the Jinja2 expressions.
        type: list
        elements: str
        default: [default]
'''
